/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: package-info.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/

/***********************************************************************************************************************
 *
 * The Hierarchy API make it possible to create and manage hierarchic views of objects.
 * TBD
 * 
 * <pre>
 * HViewManager hvManager = Locator.find(HViewManager);
 * HView hView = hvManager.findOrCreate().withId("urn:MyView1"). // id format is provider-dependent
 *                                        withDisplayName("My View 1").
 *                                        build();
 * hView.createChild().withId("urn:MyRoot1").
 *                     withDisplayName("My Root 1").
 *                     build();
 * hView.createChild().withId("urn:MyRoot2").
 *                     withDisplayName("My Root 2").
 *                     build();
 * hView.createChild().withId("urn:MyRoot3").
 *                     withDisplayName("My Root 3").
 *                     thatCanBeRenamed().
 *                     build();
 * </pre>
 *
 *
 * <h3>Display names</h3>
 *
 * All instances of {@code HView} and {@code HItem} must have a {@link Displayable} (or {@link MutableDisplayable})
 * personality available by means of {@code as(Displayable)} and in their {@link Lookup}. It can be provided in any
 * of the following ways:
 *
 * <ol>
 * <li>explicitly, by calling the {@code withDisplayName()} method of the builder;</li>
 * <li>implicitly, by binding an object with the {@code Displayable} personality with {@code boundTo()}.</li>
 * </ol>
 *
 * In all the other cases, an {@link IllegalArgumentException} will be thrown.
 * 
 *
 * <h3>IdFactories</h3>
 *
 * When an explicit id for a child is not provided, each parent uses its own {@link IdFactory} (contained in its
 * {@link Lookup}. If not specified, the parent {@link IdFactory} is used, and if not present in the whole hierarchy,
 * the one in the {@link HViewManager} is used. It is mandatory for the latter class to own an
 * {@code IdFactory} - where it's injected by the {@link LookupFactory}.
 *
 * <pre>
 * IdFactory idFactory = ...;
 * HViewManager hvManager = Locator.find(HViewManager);
 * HView hView = hvManager.findOrCreate().withId("urn:MyView1").
 *                                        withDisplayName("My View 1").
 *                                        withIdFactory(idFactory).
 *                                        build();
 * </pre>
 *
 * @author Fabrizio Giudici
 * @version $Id: package-info.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.hierarchy;

