/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: HViewTestSupport.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.hierarchy;

import it.tidalwave.hierarchy.HItem;
import it.tidalwave.hierarchy.HView;
import it.tidalwave.util.NotFoundException;
import org.junit.Test;
import static org.junit.Assert.*;
import static org.hamcrest.CoreMatchers.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: HViewTestSupport.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public abstract class HViewTestSupport extends HViewHItemCommonTestSupport<HView>
  {
    @Test
    public void aDefaultItemMustReturnTheRightView()
      {
        final HItem item = fixture.createChild().
                                   withDisplayName("My Item 1").
                                   build();
        assertThat(item, is(notNullValue()));
        assertThat(item.getView(), is(sameInstance(fixture)));
      }

    @Test
    public void aRenamableItemMustReturnTheRightView()
      {
        final HItem item = fixture.createChild().
                                   withDisplayName("My Item 1").
                                   thatCanBeRenamed().
                                   build();
        assertThat(item, is(notNullValue()));
        assertThat(item.getView(), is(sameInstance(fixture)));
      }

    @Test(expected=NotFoundException.class)
    public void aRootItemMustHaveNoParent()
      throws NotFoundException
      {
        final HItem item = fixture.createChild().
                                   withDisplayName("My Item 1").
                                   build();
        assertThat(item, is(notNullValue()));
        final HItem parent = item.getParent();
        fail("Returned parent: " + parent);
      }
  }