/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: UsagesTestSupport.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.hierarchy;

import javax.annotation.Nonnull;
import it.tidalwave.netbeans.util.Locator;
import static it.tidalwave.hierarchy.HViewManager.HViewManager;
import org.junit.BeforeClass;
import org.junit.Test;
import it.tidalwave.netbeans.util.test.MockLookup;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: UsagesTestSupport.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public class UsagesTestSupport
  {
    private static class Photo
      {
        @Nonnull
        private final String name;

        public Photo (final @Nonnull String name)
          {
            this.name = name;
          }

        @Override @Nonnull
        public String toString()
          {
            return String.format("Photo[%s]");
          }
      }

    @BeforeClass
    public static void setupIdFactory()
      {
        MockLookup.setInstances(new MockIdFactoryDelegate());
      }

    @Test
    public void scenario1()
      {
        Photo photo1 = new Photo("photo1");
        Photo photo2 = new Photo("photo2");
        Photo photo3 = new Photo("photo3");

        HViewManager hvManager = Locator.find(HViewManager);
        HView galleries = hvManager.findOrCreateView().withId("urn:Galleries").
                                                       withDisplayName("Galleries").
                                                       build();
        HItem gallery1 = galleries.createChild().withDisplayName("Gallery 1").build();
        HItem folder1 = gallery1.createChild().withDisplayName("Folder 1").build();

        gallery1.createChild().boundTo(photo1).build();
        folder1.createChild().boundTo(photo2).build();
        folder1.createChild().boundTo(photo3).build();

        HView trips = hvManager.findOrCreateView().withId("urn:Trips").
                                                   withDisplayName("Trips").
                                                   build();
        HItem trip1 = trips.createChild().withDisplayName("Trip 1").build();
        trip1.createChild().boundTo(photo1).build();
        trip1.createChild().boundTo(photo3).build();

        for (HItem item : gallery1.findChildren().boundToType(Photo.class).results())
          {
            System.err.println(item.as(Photo.class));
          }

        for (HView item : hvManager.findItems().boundTo(photo3).results())
          {
            System.err.println(item);
          }

        // also provide photo3.as(HierarchicalItemFinder).find().results()
      }
  }