/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: HExplorerPresentationVisualTest.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.hierarchy.explorer;

import it.tidalwave.hierarchy.HItem;
import it.tidalwave.hierarchy.HView;
import it.tidalwave.hierarchy.HViewManager;
import it.tidalwave.hierarchy.MockIdFactoryDelegate;
import static it.tidalwave.hierarchy.HViewManager.HViewManager;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import javax.swing.JFrame;
import javax.swing.WindowConstants;
import org.openide.explorer.ExplorerManager;
import org.openide.explorer.view.BeanTreeView;
import org.junit.Test;
import it.tidalwave.netbeans.nodes.role.RootNodeProvider;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.netbeans.util.test.MockLookup;
import org.junit.BeforeClass;
import static org.junit.Assert.*;
import static org.hamcrest.CoreMatchers.*;

class HExplorerTestFrame extends JFrame implements ExplorerManager.Provider
  {
    private final ExplorerManager explorerManager = new ExplorerManager();
    private final BeanTreeView treeView = new BeanTreeView();

    public HExplorerTestFrame()
      throws NotFoundException
      {
        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        final Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width - 400) / 2, (screenSize.height - 600) / 2, 400, 600);
        getContentPane().add(treeView, BorderLayout.CENTER);
        final HViewManager hViewManager = Locator.find(HViewManager.class);
        explorerManager.setRootContext(hViewManager.as(RootNodeProvider.class).getRootNode());
      }

    @Override
    public ExplorerManager getExplorerManager()
      {
        return explorerManager;
      }
  }

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: HExplorerPresentationVisualTest.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public class HExplorerPresentationVisualTest
  {
    @BeforeClass
    public static void setupIdFactory()
      {
        MockLookup.setInstances(new MockIdFactoryDelegate());
      }

    @Test
    public void dummy()
      {
      }

    @Test
    public void testIntegrationVisually()
      throws Exception
      {
        final HViewManager hViewManager = Locator.find(HViewManager);
        assertThat(hViewManager,is(not(nullValue())));

        HView galleryView = hViewManager.createView().withDisplayName("Galleries").build();
        HView tagsView = hViewManager.createView().withDisplayName("Tags").build();
        HView tripsView = hViewManager.createView().withDisplayName("Trips").build();
        
        final HItem[] galleries = new HItem[4];
        galleries[0] = galleryView.createChild().withDisplayName("Gallery 1").build();
        galleries[1] = galleryView.createChild().withDisplayName("Gallery 2").build();
        galleries[2] = galleryView.createChild().withDisplayName("Gallery 3").build();
        galleries[3] = galleryView.createChild().withDisplayName("Gallery 4").build();

        galleries[0].createChild().withDisplayName("Folder 1").build();
        galleries[0].createChild().withDisplayName("Folder 2").build();
        galleries[0].createChild().withDisplayName("Folder 3").build();

        final HItem[] tags = new HItem[4];
        tags[0] = tagsView.createChild().withDisplayName("Tag 1").build();
        tags[1] = tagsView.createChild().withDisplayName("Tag 2").build();
        tags[2] = tagsView.createChild().withDisplayName("Tag 3").build();
        tags[3] = tagsView.createChild().withDisplayName("Tag 4").build();

        final HItem[] trips = new HItem[4];
        trips[0] = tripsView.createChild().withDisplayName("Trip 1").build();
        trips[1] = tripsView.createChild().withDisplayName("Trip 2").build();
        trips[2] = tripsView.createChild().withDisplayName("Trip 3").build();
        trips[3] = tripsView.createChild().withDisplayName("Trip 4").build();
        
        final HExplorerTestFrame frame = new HExplorerTestFrame();
        frame.setVisible(true);
        long time = System.currentTimeMillis();
        
        while (frame.isVisible() && (System.currentTimeMillis() - time < 10000))
          {
            Thread.sleep(100);  
          }
      }
  }