/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: HViewNode.java,v ad650f4b8a8a 2010/04/04 21:30:37 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.hierarchy.node;

import javax.annotation.Nonnull;
import java.util.List;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import it.tidalwave.util.Task;
import it.tidalwave.hierarchy.HView;
import it.tidalwave.netbeans.aspect.Aspects;
import it.tidalwave.netbeans.nodes.EnhancedNode;
import it.tidalwave.netbeans.nodes.SafeChildFactory;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: HViewNode.java,v ad650f4b8a8a 2010/04/04 21:30:37 fabrizio $
 *
 **********************************************************************************************************************/
public class HViewNode extends EnhancedNode
  {
    static class HChildFactory extends SafeChildFactory<HView>
      {
        @Nonnull
        private final HView hView;

        public HChildFactory (final @Nonnull HView hView)
          {
            this.hView = hView;
          }

        @Override @Nonnull
        protected Node createNodeForKey (final @Nonnull HView hView)
          {
            // FIXME: try to move Aspects elsewhere, e.g. to EnhancedNode
            return Aspects.run(hView, new Task<Node, RuntimeException>("createNodeForKey()")
              {
                @Override @Nonnull
                public Node run()
                  {
                    return new HViewNode(hView);
                  }
              });
          }

        @Override
        protected boolean createKeysSafely (final @Nonnull List<HView> keys)
          {
            // FIXME: try to move Aspects elsewhere, e.g. to EnhancedNode
            return Aspects.run(hView, new Task<Boolean, RuntimeException>("createKeysSafely()")
              {
                @Override @Nonnull
                public Boolean run()
                  {
                    keys.addAll(hView.findChildren().results());
                    return true;
                  }
              });
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public HViewNode (final @Nonnull HView hView)
      {
        super(hView, Children.create(new HChildFactory(hView), true),
                     new ProxyLookup(hView.getLookup(), Lookups.fixed(hView)));
      }
  }
