/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: HViewManagerNodeTest.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.hierarchy.node;

import java.util.Arrays;
import org.openide.nodes.Node;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.hierarchy.HView;
import it.tidalwave.hierarchy.HViewManager;
import static it.tidalwave.hierarchy.HViewManager.HViewManager;
import it.tidalwave.hierarchy.MockIdFactoryDelegate;
import it.tidalwave.netbeans.util.test.MockLookup;
import it.tidalwave.netbeans.util.test.Resettable;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;
import static org.hamcrest.CoreMatchers.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: HViewManagerNodeTest.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public class HViewManagerNodeTest
  {
    private HViewManager hViewManager;
    private final HView hViews[] = new HView[3];

    @BeforeClass
    public static void setIdFactory()
      {
        MockLookup.setInstances(new MockIdFactoryDelegate());
      }

    @Before
    public void setUp()
      {
        hViewManager = Locator.find(HViewManager);
        hViewManager.as(Resettable.class).reset();
        assertThat(hViewManager.findViews().count(), is(0));
        hViews[0] = hViewManager.createView().withDisplayName("view 1").build();
        hViews[1] = hViewManager.createView().withDisplayName("view 2").build();
        hViews[2] = hViewManager.createView().withDisplayName("view 3").build();
      }

    @Test(timeout=3000)
    public void mustCreateCorrectChildren()
      throws InterruptedException
      {
        final HViewManagerNode hViewManagerNode = new HViewManagerNode(hViewManager);
        assertThat(hViewManagerNode.canCopy(), is(false));
        assertThat(hViewManagerNode.canCut(), is(false));
        assertThat(hViewManagerNode.canDestroy(), is(false));
        assertThat(hViewManagerNode.canRename(), is(false));

        assertThat(hViewManagerNode.getLookup().lookup(HViewManager), is(sameInstance(hViewManager)));

        assertThat(hViewManagerNode.getDisplayName(), is("Hierarchic Views"));
        Node[] children = hViewManagerNode.getChildren().getNodes();

        while (children[0].getClass().getName().contains("WaitFilterNode"))
          {
            Thread.sleep(100);
            children = hViewManagerNode.getChildren().getNodes();
          }

        assertThat(children, is(notNullValue()));
        assertThat("" + Arrays.toString(children), children.length, is(hViews.length));
        
        for (int i = 0; i < children.length; i++)
          {
            assertThat(children[i].getDisplayName(), is("view " + (i + 1)));
            assertThat(children[i].getLookup().lookup(HView.class), is(sameInstance(hViews[i])));
          }
      }
  }