/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: HViewNodeTest.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.hierarchy.node;

import org.openide.nodes.Node;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.hierarchy.HItem;
import it.tidalwave.hierarchy.HView;
import it.tidalwave.hierarchy.HViewManager;
import it.tidalwave.hierarchy.MockIdFactoryDelegate;
import static it.tidalwave.hierarchy.HViewManager.HViewManager;
import static it.tidalwave.hierarchy.HItem.HItem;
import static it.tidalwave.hierarchy.HView.HView;
import it.tidalwave.netbeans.util.test.MockLookup;
import it.tidalwave.netbeans.util.test.Resettable;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;
import static org.hamcrest.CoreMatchers.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: HViewNodeTest.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public class HViewNodeTest
  {
    private HViewManager hViewManager;
    private HView hView;
    private final HItem hItems[] = new HItem[3];

    @BeforeClass
    public static void setIdFactory()
      {
        MockLookup.setInstances(new MockIdFactoryDelegate());
      }

    @Before
    public void setUp()
      {
        hViewManager = Locator.find(HViewManager);
        hViewManager.as(Resettable.class).reset();
        hView = hViewManager.createView().withDisplayName("view 1").build();
        hItems[0] = hView.createChild().withDisplayName("item 1").build();
        hItems[1] = hView.createChild().withDisplayName("item 2").build();
        hItems[2] = hView.createChild().withDisplayName("item 3").build();
      }

    @Test(timeout=3000)
    public void mustCreateCorrectChildren()
      throws InterruptedException
      {
        final HViewNode hViewNode = new HViewNode(hView);

        assertThat(hViewNode.getLookup().lookup(HView), is(sameInstance(hView)));

        assertThat(hViewNode.getDisplayName(), is("view 1"));
        Node[] children = hViewNode.getChildren().getNodes();

        while (children[0].getClass().getName().contains("WaitFilterNode"))
          {
            Thread.sleep(100);
            children = hViewNode.getChildren().getNodes();
          }

        assertThat(children, is(notNullValue()));
        assertThat(children.length, is(hItems.length));
        
        for (int i = 0; i < children.length; i++)
          {
            assertThat(children[i].getDisplayName(), is("item " + (i + 1)));
            assertThat(children[i].getLookup().lookup(HItem), is(sameInstance(hItems[i])));
          }
      }
  }