/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: ByteCountProgressPanel.java,v 6faa0e6eda01 2010/03/28 22:55:32 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.downloadprogresspane;

import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import org.jdesktop.beansbinding.AutoBinding;
import org.jdesktop.beansbinding.BeanProperty;
import org.jdesktop.beansbinding.BindingGroup;
import org.jdesktop.beansbinding.Bindings;
//import it.tidalwave.util.parameters.Parameters;

/***********************************************************************************************************************
 *
 * A convenience panel which shows a progress panel describing a progress
 * expressed in byte count, useful for read/write tasks.
 *
 * @author  Fabrizio Giudici
 * @version $Id: ByteCountProgressPanel.java,v 6faa0e6eda01 2010/03/28 22:55:32 fabrizio $
 *
 **********************************************************************************************************************/
public class ByteCountProgressPanel extends javax.swing.JPanel
  {
    public static final String PROP_PROCESSED_BYTE_COUNT = "processedByteCount";
    public static final String PROP_TOTAL_BYTE_COUNT = "totalByteCount";
    private static final long serialVersionUID = 8922284261666285422L;
    
    protected int processedByteCount;
    protected int totalByteCount;

    /***************************************************************************
     *
     *
     **************************************************************************/
    public ByteCountProgressPanel()
      {
        initComponents();
        pbProgressBar.setIndeterminate(true);
      }

    /***************************************************************************
     *
     * Convenience method that binds the relevant properties of this panel to
     * some source. Remember to <code>unbind()</code> the returned bindings
     * when they are no more used.
     *
     * @param  source                     the bean which will set the values
     * @param  processedByteCountPropertyName  the name of the property which
     *                                    describes the progress count
     * @param  totalByteCountPropertyName the name of the property which
     *                                    describes the total byte count
     * @return                            the bindings
     *
     **************************************************************************/
    @Nonnull
    public BindingGroup bind (final @Nonnull Object source,
                              final @Nonnull String processedByteCountPropertyName,
                              final @Nonnull String totalByteCountPropertyName)
      {
//        Parameters.checkNonNull(source, "source"); FIXME
//        Parameters.checkNonNull(processedByteCountPropertyName, "processedByteCountPropertyName");
//        Parameters.checkNonNull(totalByteCountPropertyName, "totalByteCountPropertyName");

        final BindingGroup bindings = new BindingGroup();
        bindings.addBinding(Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ,
                                                       source,
                                                       BeanProperty.create(processedByteCountPropertyName),
                                                       this,
                                                       BeanProperty.create(PROP_PROCESSED_BYTE_COUNT)));
        bindings.addBinding(Bindings.createAutoBinding(AutoBinding.UpdateStrategy.READ,
                                                       source,
                                                       BeanProperty.create(totalByteCountPropertyName),
                                                       this,
                                                       BeanProperty.create(PROP_TOTAL_BYTE_COUNT)));
        bindings.bind();

        return bindings;
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    public void setProcessedByteCount (@Nonnegative final int processedByteCount)
      {
        int oldProcessedByteCount = this.processedByteCount;
        this.processedByteCount = processedByteCount;
        firePropertyChange(PROP_PROCESSED_BYTE_COUNT, oldProcessedByteCount, processedByteCount);
        pbProgressBar.setValue(processedByteCount);
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    @Nonnegative
    public int getProcessedByteCount()
      {
        return processedByteCount;
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    public void setTotalByteCount (@Nonnegative final int totalByteCount)
      {
        final int oldTotalByteCount = this.totalByteCount;
        this.totalByteCount = totalByteCount;
        firePropertyChange(PROP_TOTAL_BYTE_COUNT, oldTotalByteCount, totalByteCount);
        pbProgressBar.setMaximum(totalByteCount);
        pbProgressBar.setIndeterminate(false);
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    @Nonnegative
    public int getTotalByteCount()
      {
        return totalByteCount;
      }

    /***************************************************************************
     *
     * This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     *
     **************************************************************************/
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        bindingGroup = new org.jdesktop.beansbinding.BindingGroup();

        jPanel1 = new javax.swing.JPanel();
        lbLoaded = new javax.swing.JLabel();
        pbProgressBar = new javax.swing.JProgressBar();
        lbBytes = new javax.swing.JLabel();

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder(""));

        lbLoaded.setFont(lbLoaded.getFont().deriveFont(lbLoaded.getFont().getSize()-2f));
        lbLoaded.setText(org.openide.util.NbBundle.getMessage(ByteCountProgressPanel.class, "ByteCountProgressPanel.lbLoaded.text")); // NOI18N

        lbBytes.setFont(lbBytes.getFont().deriveFont(lbBytes.getFont().getSize()-2f));

        org.jdesktop.beansbinding.Binding binding = org.jdesktop.beansbinding.Bindings.createAutoBinding(org.jdesktop.beansbinding.AutoBinding.UpdateStrategy.READ_WRITE, this, org.jdesktop.beansbinding.ELProperty.create("${processedByteCount} bytes"), lbBytes, org.jdesktop.beansbinding.BeanProperty.create("text"));
        binding.setSourceNullValue("0 bytes");
        binding.setSourceUnreadableValue("0 bytes");
        bindingGroup.addBinding(binding);

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(pbProgressBar, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 395, Short.MAX_VALUE)
                    .add(jPanel1Layout.createSequentialGroup()
                        .add(lbLoaded)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(lbBytes, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 173, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(pbProgressBar, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lbLoaded)
                    .add(lbBytes, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 16, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );

        bindingGroup.bind();
    }// </editor-fold>//GEN-END:initComponents

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel jPanel1;
    private javax.swing.JLabel lbBytes;
    private javax.swing.JLabel lbLoaded;
    private javax.swing.JProgressBar pbProgressBar;
    private org.jdesktop.beansbinding.BindingGroup bindingGroup;
    // End of variables declaration//GEN-END:variables
  }
