/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: DataObjectPasteType.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.draganddrop;

import javax.annotation.Nonnull;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.io.IOException;
import org.openide.util.datatransfer.PasteType;
import org.openide.loaders.DataNode;
import org.openide.loaders.DataObject;

/***********************************************************************************************************************
 *
 * A PasteType which enable pasting to this node, delegating the behaviour to a plugged class.
 * 
 * @author  Fabrizio Giudici
 * @version $Id: DataObjectPasteType.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public class DataObjectPasteType extends PasteType 
  {
    public static final DataFlavor PASTE_ACTION_FLAVOR = new DataFlavor(DataObjectPasteType.Action.class, "PasteAction");

    /*******************************************************************************************************************
     *
     * An interface for implementing pluggable paste behaviour. Extension code
     * just has to implement a Transferable which supports a PASTE_ACTION_FLAVOR
     * and returns a implementation of PasteAction.
     *
     ******************************************************************************************************************/
    public static interface Action 
      { 
        public void paste (@Nonnull DataObject dataObject);
      }
    
    @Nonnull
    private final Action pasteAction;
    
    @Nonnull
    private final DataNode node;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public DataObjectPasteType (@Nonnull final Action pasteAction, 
                                @Nonnull final DataNode node) 
      {
        super();
        this.node = node;
        this.pasteAction = pasteAction;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public String getDisplayName() 
      {
        return "Paste tag...";// FIXME
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public Transferable paste()
      throws IOException 
      {
        pasteAction.paste(node.getDataObject());
        return null;
      }
}
