/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: TransferableSupport.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.draganddrop;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;import java.util.HashMap;
import java.util.Map;
import java.io.IOException;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;


/***********************************************************************************************************************
 *
 * A Node subclass for representing a Folder.
 *
 * @author  Fabrizio Giudici
 * @version $Id: TransferableSupport.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public class TransferableSupport implements Transferable
  {
    private final Map<DataFlavor, Object> objectMapByDataFlavor = new HashMap<DataFlavor, Object>();

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void addObject (final @Nonnull DataFlavor dataFlavor,
                           final @Nonnull Object object)
      {
        objectMapByDataFlavor.put(dataFlavor, object);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    @Nonnull
    public DataFlavor[] getTransferDataFlavors() 
      {
        return objectMapByDataFlavor.keySet().toArray(new DataFlavor[0]);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    public boolean isDataFlavorSupported (final @Nonnull DataFlavor flavor)
      {
        return objectMapByDataFlavor.containsKey(flavor);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    @CheckForNull
    public Object getTransferData (final @Nonnull DataFlavor flavor)
      throws UnsupportedFlavorException, IOException 
      {
        if (!objectMapByDataFlavor.containsKey(flavor))
          {
            throw new UnsupportedFlavorException(flavor);
          }

        return objectMapByDataFlavor.get(flavor);
      }
  }
