/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: IndexedDataObjectList.java,v 4e8a394a80a9 2010/03/28 20:51:11 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.indexeddataobject;

import java.util.AbstractList;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.io.IOException;
import org.openide.loaders.DataObject;
import it.tidalwave.util.Identifiable;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.util.Id;

/***********************************************************************************************************************
 *
 * This special implementation of {@link List} holds a collection of {@link IndexedDataObject} without keeping direct 
 * references to them; instead their ids are stored.
 *
 * TODO: check if you can rather held the serialized bytes; this would work the same for indexed DataObjects, but you
 * could drop the dependency on IndexedDataObjectManager and move to OpenBlueSky - EnhancedDataObject.
 * EVEN BETTER, you could define an injectable Handle role and use it!
 * 
 * @author  Fabrizio Giudici
 * @version $Id: IndexedDataObjectList.java,v 4e8a394a80a9 2010/03/28 20:51:11 fabrizio $
 *
 **********************************************************************************************************************/
public class IndexedDataObjectList<T extends DataObject> extends AbstractList<T>
  {
    private final IndexedDataObjectManager dataObjectManager = Locator.find(IndexedDataObjectManager.class);
    
    private final List<Id> ids = new ArrayList<Id>();

    /***************************************************************************
     * 
     *
     **************************************************************************/
    public IndexedDataObjectList()
      {
      }
    
    /***************************************************************************
     * 
     *
     **************************************************************************/
    public IndexedDataObjectList (final Collection<T> collection)
      {
        addAll(collection);
      }
    
    /***************************************************************************
     * 
     * {@inheritDoc}
     *
     **************************************************************************/
    @Override
    public void add (final int index, final T element) 
      {
        ids.add(index, element.getLookup().lookup(Identifiable.class).getId()); 
      }

    /***************************************************************************
     * 
     * {@inheritDoc}
     *
     **************************************************************************/
    @Override
    public T remove (int index) 
      {
        final T result = get(index);
        ids.remove(index);
        return result;
      }

    /***************************************************************************
     * 
     * {@inheritDoc}
     *
     **************************************************************************/
    @Override
    public T get (int index) 
      {
        try 
          {
            return (T)dataObjectManager.findById(ids.get(index));
          } 
        catch (IOException e) 
          {
            throw new RuntimeException(e);
          }
      }   

    /***************************************************************************
     * 
     * {@inheritDoc}
     *
     **************************************************************************/
    @Override
    public int size() 
      {
        return ids.size();
      }
  }
