/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: IndexedDataObjectManager.java,v 4e8a394a80a9 2010/03/28 20:51:11 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.indexeddataobject;

import javax.annotation.Nonnull;
import java.io.IOException;
import java.util.Collection;
import java.util.List;
import org.openide.loaders.DataObject;
import it.tidalwave.util.Id;

/***********************************************************************************************************************
 *
 * The <code>IndexedDataObjectManager</code> is a service which allows to bind
 * <code>DataObject</code>s with an unique id. This enables two operations:
 * retrieving the id from a <code>DataObject</code> and retrieving a
 * <code>DataObject</code> giving a its id.<br>
 *
 * @author  Fabrizio Giudici
 * @version $Id: IndexedDataObjectManager.java,v 4e8a394a80a9 2010/03/28 20:51:11 fabrizio $
 *
 **********************************************************************************************************************/
public interface IndexedDataObjectManager 
  {
    /***************************************************************************
     * 
     * Retrieves a <code>DataObject</code> given its id.
     * 
     * @param  id           the id
     * @return              the <code>DataObject</code> or null if no object is found
     * @throws IOException  if an error occurred 
     *
     **************************************************************************/
    public DataObject findById (@Nonnull Id id)
      throws IOException;
    
    // this optimizes batches
    public List<DataObject> findByIds (@Nonnull Collection<Id> ids);
  }
