/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: ChangeDecorator.java,v ab5e96a57544 2010/04/04 20:12:06 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.indexedfilesystem.impl;

import javax.annotation.Nonnull;
import java.io.IOException;
import org.openide.filesystems.AbstractFileSystem;
import org.openide.filesystems.FileObject;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.netbeans.indexedfilesystem.spi.FileIndexer;

/***********************************************************************************************************************
 *
 * This class intercepts renames and updates indexing for folder contents.
 *
 * @author  Fabrizio Giudici
 * @version $Id: ChangeDecorator.java,v ab5e96a57544 2010/04/04 20:12:06 fabrizio $
 *
 **********************************************************************************************************************/
public class ChangeDecorator implements AbstractFileSystem.Change
  {
    private static final String CLASS = ChangeDecorator.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);
    private static final long serialVersionUID = -2403111710853967073L;

    private static final FileIndexer FILE_INDEXER = Locator.find(FileIndexer.class);

    @Nonnull
    private final AbstractFileSystem.Change delegate;


    @Nonnull
    private final LocalIndexedFileSystem fileSystem;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    ChangeDecorator (final @Nonnull AbstractFileSystem.Change delegate,
                     final @Nonnull LocalIndexedFileSystem fileSystem)
      {
        this.delegate = delegate;
        this.fileSystem = fileSystem;
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    public void rename (final @Nonnull String oldName, final @Nonnull String newName)
      throws IOException
      {
        logger.finest("rename(%s -> %s)", oldName, newName);

        final FileObject fileObject = fileSystem.findResource(oldName);

        if (fileObject.isFolder())
          {
            FILE_INDEXER.renameRecursively(fileObject, oldName, newName);
          }

        delegate.rename(oldName, newName);
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    public void delete (final @Nonnull String name)
      throws IOException
      {
        logger.finest("delete(%s)", name);
        delegate.delete(name);
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    public void createFolder (final @Nonnull String name)
      throws IOException
      {
        delegate.createFolder(name);
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    public void createData (final @Nonnull String name)
      throws IOException
      {
        delegate.createData(name);
      }
  }
