/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: ListDecorator.java,v ab5e96a57544 2010/04/04 20:12:06 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.indexedfilesystem.impl;

import javax.annotation.Nonnull;
import org.openide.filesystems.AbstractFileSystem;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.netbeans.indexedfilesystem.spi.FileIndexer;

/***********************************************************************************************************************
 *
 * This class intercepts scanning for children files in order to pre-load the normalizedPathPendingList - this will
 * improve performance allowing for batch loading of bindings.
 *
 * @author  Fabrizio Giudici
 * @version $Id: ListDecorator.java,v ab5e96a57544 2010/04/04 20:12:06 fabrizio $
 *
 **********************************************************************************************************************/
class ListDecorator implements AbstractFileSystem.List
  {
    private static final String CLASS = ListDecorator.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);
    private static final long serialVersionUID = 3533859919153986028L;
    
    private static final FileIndexer FILE_INDEXER = Locator.find(FileIndexer.class);

    @Nonnull
    private final AbstractFileSystem.List delegate;

    @Nonnull
    private final LocalIndexedFileSystem fileSystem;
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public ListDecorator (final @Nonnull AbstractFileSystem.List delegate,
                          final @Nonnull LocalIndexedFileSystem fileSystem)
      {
        this.delegate = delegate;
        this.fileSystem = fileSystem;
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public String[] children (final @Nonnull String path)
      {
        final String[] result = delegate.children(path);
        
        if (result != null)
          {
            synchronized (fileSystem.DISABLE_CHILDREN_INDEXING_LOCK)
              {
                if (!fileSystem.disableChildrenIndexing)
                  {
                    logger.fine("children(%s)", path);
                    String path2 = fileSystem.findCompletePath(path);
                    if (!path2.endsWith("/") && !path2.endsWith("\\"))
                      {
                        path2 = path2 + "/";
                      }
                    
                    logger.finer(">>> fixed path: %s", path2);

                    for (final String child : result)
                      {
                        FILE_INDEXER.createIndexing(path2 + child, true);
                      }
                  }
              }
          }

        return result;
      }
  }
