/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: RootFileObject.java,v 9389d0054e4c 2010/04/04 20:17:26 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.indexedfilesystem.impl;

import javax.annotation.Nonnull;
import java.util.Date;
import java.util.Enumeration;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import org.openide.filesystems.FileChangeListener;
import org.openide.filesystems.FileLock;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.FileUtil;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: RootFileObject.java,v 9389d0054e4c 2010/04/04 20:17:26 fabrizio $
 *
 **********************************************************************************************************************/
public class RootFileObject extends FileObject
  {
    private static final long serialVersionUID = -4202460482095260534L;

    @Nonnull
    private final FileSystem fileSystem;

    @Nonnull
    private final FileObject[] children;
    
    public RootFileObject (final @Nonnull FileSystem fileSystem)
      {
        this.fileSystem = fileSystem;
        final File[] roots = File.listRoots();
        children = new FileObject[roots.length];
        
        for (int i = 0; i < roots.length; i++)
          {
            children[i] = FileUtil.toFileObject(roots[i]);  
          }
      }

    @Override @Nonnull
    public String getName() 
      {
        return "My computer";
      }

    @Override @Nonnull
    public String getExt() 
      {
        return "";
      }

    @Override @Nonnull
    public void rename (final @Nonnull FileLock fileLock, final @Nonnull String string, final @Nonnull String string0)
      throws IOException 
      {
        throw new UnsupportedOperationException("Can't do on root");
      }

    @Override @Nonnull
    public FileSystem getFileSystem() 
      {
        return fileSystem;
      }

    @Override
    public FileObject getParent() 
      {
        return null;
      }

    @Override
    public boolean isFolder() 
      {
        return true;
      }

    @Override @Nonnull
    public Date lastModified() 
      {
        return new Date(0);
      }

    @Override
    public boolean isRoot() 
      {
        return true;
      } 

    @Override
    public boolean isData() 
      {
        return false;
      }

    @Override
    public boolean isValid() 
      {
        return true;
      }

    @Override
    public void delete (final @Nonnull FileLock fileLock)
      {
        throw new UnsupportedOperationException("Can't do on root");
      }

    @Override
    public Object getAttribute (final @Nonnull String string)
      {
        return null;
      }

    @Override
    public void setAttribute (final @Nonnull String string, final @Nonnull Object object)
      {
        throw new UnsupportedOperationException("Can't do on root");
      }

    @Override @Nonnull
    public Enumeration getAttributes() 
      {
        return new Enumeration() 
          {
            @Override
            public boolean hasMoreElements() 
              {
                return false;
              }
            
            @Override
            public Object nextElement() 
              {
                throw new NoSuchElementException();
              }
          };
      }

    @Override
    public void addFileChangeListener (final @Nonnull FileChangeListener fileChangeListener)
      {
        throw new UnsupportedOperationException("Can't do on root");
      }

    @Override
    public void removeFileChangeListener (FileChangeListener fileChangeListener)
      {
        throw new UnsupportedOperationException("Can't do on root");
      }

    @Override
    public long getSize() 
      {
        return 0;
      }

    @Override @Nonnull
    public InputStream getInputStream() 
      throws FileNotFoundException 
      {
        throw new FileNotFoundException("Can't do on root");
      }

    @Override @Nonnull
    public OutputStream getOutputStream (FileLock fileLock)
      throws IOException
      {
        throw new FileNotFoundException("Can't do on root");
      }

    @Override @Nonnull
    public FileLock lock() 
      {
        throw new UnsupportedOperationException("Can't do on root");
      }

    @Override
    public void setImportant (boolean b)
      {
      }

    @Override @Nonnull
    public FileObject[] getChildren() 
      {
        return children;
      }

    @Override @Nonnull
    public FileObject getFileObject (final @Nonnull String name, final @Nonnull String ext)
      {
        for (FileObject child : children)
          {
            if (name.equals(child.getName()))
              {
                return child;  
              }
          }
        
        return null;
      }

    @Override @Nonnull
    public FileObject createFolder (final @Nonnull String string)
      {
        throw new UnsupportedOperationException("Can't do on root");
      }

    @Override @Nonnull
    public FileObject createData (final @Nonnull String string, final @Nonnull String string0)
      {
        throw new UnsupportedOperationException("Can't do on root");
      }

    @Override
    public boolean isReadOnly() 
      {
        return false;
      }

    @Nonnull
    protected FileObject find (final @Nonnull String path)
      {
        StringTokenizer st = new StringTokenizer(path, File.separator);
        FileObject fo = this;
        
        while (st.hasMoreTokens())
          {
            String s = st.nextToken();  
            fo = fo.getFileObject(s, "");
          }

        return fo;
      }
  }
