/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: TestPersistenceConfigurator.java,v 242fd62b9818 2010/04/05 00:12:57 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.indexedfilesystem.impl;

import javax.annotation.Nonnull;
import java.util.Properties;
import java.sql.DriverManager;
import org.openide.util.lookup.ServiceProvider;
import it.tidalwave.netbeans.persistence.spi.PersistenceConfigurator;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: TestPersistenceConfigurator.java,v 242fd62b9818 2010/04/05 00:12:57 fabrizio $
 *
 **********************************************************************************************************************/
@ServiceProvider(service=PersistenceConfigurator.class)
public class TestPersistenceConfigurator implements PersistenceConfigurator
  {
    private String databaseURL;
    private String databaseType = "Derby";

    private boolean closed;

    public boolean isClosed() 
      {
        return closed;
      }

    @Override @Nonnull
    public String getPersistenceUnitName()
      {
        return "TEST_PU";
      }

    @Override @Nonnull
    public Properties initialize()
      {
        final Properties properties = new Properties();

        if ("Derby".equals(databaseType))
          {
            databaseURL = "jdbc:derby:target/test-artifacts/TestDatabase";
//            databaseURL = "jdbc:derby:" + System.getProperty("derby.folder");
            properties.setProperty("hibernate.connection.driver_class", "org.apache.derby.jdbc.EmbeddedDriver");
            properties.setProperty("hibernate.connection.url", databaseURL + ";create=true");
            properties.setProperty("connection.driver_class", "org.apache.derby.jdbc.EmbeddedDriver");
            properties.setProperty("connection.url", databaseURL + ";create=true");
            properties.setProperty("hibernate.connection.username", "");
            properties.setProperty("hibernate.connection.password", "");
            properties.setProperty("hibernate.dialect", "org.hibernate.dialect.DerbyDialect");
          }
        else if ("Postgres".equals(databaseType))
          {
            databaseURL = "jdbc:postgresql://localhost:5432/bluemarine_ci";
            properties.setProperty("hibernate.connection.driver_class", "org.postgresql.Driver");
            properties.setProperty("hibernate.connection.url", databaseURL);
            properties.setProperty("connection.driver_class", "org.postgresql.Driver");
            properties.setProperty("connection.url", databaseURL);
            properties.setProperty("hibernate.connection.username", "postgres");
            properties.setProperty("hibernate.connection.password", "");
            properties.setProperty("hibernate.dialect", "org.hibernate.dialect.PostgreSQLDialect");
          }

        return properties;
      }

    @Override
    public void shutdown()
      {
        closed = true;
        
        if ("Derby".equals(databaseType) && (databaseURL != null))
          {
            try
              {
                DriverManager.getConnection(databaseURL + ";shutdown=true");
              }
            catch (Exception e)
              {
              }
          }
      }
  }
