/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: LicenseModule.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.licensepanel;

import java.util.prefs.Preferences;
import org.openide.modules.ModuleInstall;
import org.openide.LifecycleManager;
import org.openide.util.NbBundle;
import org.openide.util.Utilities;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.netbeans.dialog.DialogRunner;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: LicenseModule.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public class LicenseModule extends ModuleInstall
  {
    private static final String CLASS = LicenseModule.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);
    
    /** Change this when you want to force a license display in installed versions. */
    private static final String PROP = NbBundle.getMessage(LicenseModule.class, "License.licenseKey");
    
    private static final String PREFERENCES_NODE =
            NbBundle.getMessage(LicenseModule.class, "License.licensePreferencePath");
    
    private static final String CHANGE_ME = "CHANGE-ME";
    
    @Override
    public void restored() 
      {
        if (PROP.contains(CHANGE_ME) || PREFERENCES_NODE.contains(CHANGE_ME))
          {
            final String message = NbBundle.getMessage(LicenseModule.class, "LicenseModule.notBrandedMessage");
            final String title = NbBundle.getMessage(LicenseModule.class, "LicenseModule.notBrandedTitle");
            logger.severe(message);
            DialogRunner.showMessage(message, title);
            LifecycleManager.getDefault().exit();
          }

        if (Utilities.isMac() || Utilities.isUnix()) // On Windows the license is shown by the installer
          {
            // Don't use NbPreferences since it is workspace-related; we want a global, per-user setting.
            final Preferences preferences = Preferences.userRoot().node(PREFERENCES_NODE);

            if (!preferences.getBoolean(PROP, false))
              {
                final LicensePane licensePane = new LicensePane();
                DialogRunner.runOkModalDialog(licensePane, NbBundle.getMessage(LicenseModule.class, "License.license"));

                if (!licensePane.isAccepted())
                  {
                    final String message = NbBundle.getMessage(LicenseModule.class, "LicenseModule.notAcceptedMessage");
                    final String title = NbBundle.getMessage(LicenseModule.class, "LicenseModule.notAcceptedTitle");
                    logger.severe(message);
                    DialogRunner.showMessage(message, title);
                    LifecycleManager.getDefault().exit();
                  }
                else
                  {
                    preferences.putBoolean(PROP, true);  
                  }
              }
          }
      }
  }
