/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: TaggingExecutorService.java,v 4e8a394a80a9 2010/03/28 20:51:11 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.rendezvous;

import it.tidalwave.netbeans.rendezvous.RendezVous.Tag;
import java.util.List;
import java.util.concurrent.AbstractExecutorService;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.TimeUnit;
import javax.annotation.Nonnull;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: TaggingExecutorService.java,v 4e8a394a80a9 2010/03/28 20:51:11 fabrizio $
 *
 **********************************************************************************************************************/
public class TaggingExecutorService extends AbstractExecutorService
  {
    private static class TaggingRunnable implements Runnable
      {
        @Nonnull
        private final Runnable delegate;

        @Nonnull
        private final Tag tag;

        public TaggingRunnable(Runnable delegate, Tag tag)
          {
            this.delegate = delegate;
            this.tag = tag;
          }

        public void run()
          {
            try
              {
                Tag.set(tag);
                delegate.run();
              }
            finally
              {
                Tag.set(null);
              }
          }
      }

    @Nonnull
    private final ExecutorService delegate;

    public TaggingExecutorService (@Nonnull final ExecutorService delegate)
      {
        this.delegate = delegate;
      }

    public void execute (@Nonnull final Runnable command)
      {
        delegate.execute(new TaggingRunnable(command, Tag.get()));
      }

    public List<Runnable> shutdownNow()
      {
        return delegate.shutdownNow();
      }

    public void shutdown()
      {
        delegate.shutdown();
      }

    public boolean isTerminated()
      {
        return delegate.isTerminated();
      }

    public boolean isShutdown()
      {
        return delegate.isShutdown();
      }

    public boolean awaitTermination (long timeout, TimeUnit unit)
      throws InterruptedException
      {
        return delegate.awaitTermination(timeout, unit);
      }
  }
