/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: EntityManagerProxy.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.persistence;

import javax.annotation.Nonnull;
import javax.persistence.EntityManager;
import javax.persistence.EntityTransaction;
import javax.persistence.FlushModeType;
import javax.persistence.LockModeType;
import javax.persistence.Query;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: EntityManagerProxy.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public class EntityManagerProxy implements EntityManager
  {
    @Override
    public void setFlushMode (@Nonnull FlushModeType flushMode)
      {
        getEMDelegate().setFlushMode(flushMode);
      }

    @Override
    public void remove (@Nonnull Object entity)
      {
        getEMDelegate().remove(entity);
      }

    @Override
    public void refresh (@Nonnull Object entity)
      {
        getEMDelegate().refresh(entity);
      }

    @Override
    public void persist (@Nonnull Object entity)
      {
        getEMDelegate().persist(entity);
      }

    @Override @Nonnull
    public <T> T merge (@Nonnull T entity)
      {
        return getEMDelegate().merge(entity);
      }

    @Override
    public void lock (@Nonnull Object entity, @Nonnull LockModeType lock)
      {
        getEMDelegate().lock(entity, lock);
      }

    @Override
    public void joinTransaction() 
      {
        getEMDelegate().joinTransaction();
      }

    @Override
    public boolean isOpen() 
      {
        return getEMDelegate().isOpen();
      }

    @Override @Nonnull
    public EntityTransaction getTransaction() 
      {
        return getEMDelegate().getTransaction();
      }

    @Override @Nonnull
    public <T> T getReference (@Nonnull Class<T> entityClass, @Nonnull Object entity)
      {
        return getEMDelegate().getReference(entityClass, entity);
      }

    @Override @Nonnull
    public FlushModeType getFlushMode() 
      {
        return getEMDelegate().getFlushMode();
      }

    @Override @Nonnull
    public Object getDelegate() 
      {
        return getEMDelegate().getDelegate();
      }

    @Override
    public void flush() 
      {
        getEMDelegate().flush();
      }

    @Override @Nonnull
    public <T> T find (@Nonnull Class<T> entityClass, @Nonnull Object id)
      {
        return getEMDelegate().find(entityClass, id);
      }

    @Override @Nonnull
    public Query createQuery (@Nonnull String query)
      {
        return getEMDelegate().createQuery(query);
      }

    @Override @Nonnull
    public Query createNativeQuery (@Nonnull String arg0, @Nonnull String arg1)
      {
        return getEMDelegate().createNativeQuery(arg0, arg1);
      }

    @Override @Nonnull
    public Query createNativeQuery (@Nonnull String arg0, @Nonnull Class arg1)
      {
        return getEMDelegate().createNativeQuery(arg0, arg1);
      }

    @Override @Nonnull
    public Query createNativeQuery (@Nonnull String arg0)
      {
        return getEMDelegate().createNativeQuery(arg0);
      }

    @Override @Nonnull
    public Query createNamedQuery (@Nonnull String arg0)
      {
        return getEMDelegate().createNamedQuery(arg0);
      }

    @Override
    public boolean contains (@Nonnull Object entity)
      {
        return getEMDelegate().contains(entity);
      }

    @Override
    public void close() 
      {
        getEMDelegate().close();
      }

    @Override
    public void clear() 
      {
        getEMDelegate().clear();
      }

    @Nonnull
    private EntityManager getEMDelegate()
      {
        return TxTask.getEntityManager();
      }
  }
