/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: MaintainerTest.java,v 781a91131249 2010/04/05 01:20:54 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.persistence.maintenance.impl;

import java.util.Map;
import javax.persistence.Table;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.netbeans.persistence.maintenance.MaintainerTask;
import it.tidalwave.netbeans.persistence.maintenance.impl.mock.ENTITY1_v0_v1;
import it.tidalwave.netbeans.persistence.maintenance.impl.mock.ENTITY1_v1_v2;
import it.tidalwave.netbeans.persistence.maintenance.impl.mock.ENTITY2_v0_v1;
import it.tidalwave.netbeans.persistence.maintenance.impl.mock.Entity1;
import it.tidalwave.netbeans.persistence.maintenance.impl.mock.Entity2;
import it.tidalwave.netbeans.persistence.maintenance.impl.mock.OLD_ENTITY2_v0_v1;
import it.tidalwave.netbeans.persistence.maintenance.impl.mock.OLD_ENTITY2_v1_v2;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: MaintainerTest.java,v 781a91131249 2010/04/05 01:20:54 fabrizio $
 *
 **********************************************************************************************************************/
public class MaintainerTest extends MaintainerTestSupport
  {
    private static final String CLASS = MaintainerTest.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    private Maintainer maintainer;
    
    @Before
    public void setUp() 
      {
        maintainer = new Maintainer();
      }

    @After
    public void tearDown() 
      throws Exception 
      {
        maintainer.closeConnection();
        maintainer = null;
      }

    @Test
    public void testMaintainerTaskDiscovery()   
      throws Exception
      {
        logger.info("******** testMaintainerTaskDiscovery");
        assertEquals(3, maintainer.maintainerMapByPair.size());
        
        final EntityAndTablePair PAIR1 = new EntityAndTablePair(Entity1.class, "ENTITY1");
        final EntityAndTablePair PAIR2 = new EntityAndTablePair(Entity2.class, "OLD_ENTITY2");
        final EntityAndTablePair PAIR3 = new EntityAndTablePair(Entity2.class, "ENTITY2");
        
        assertTrue(maintainer.maintainerMapByPair.containsKey(PAIR1));
        assertTrue(maintainer.maintainerMapByPair.containsKey(PAIR2));
        assertTrue(maintainer.maintainerMapByPair.containsKey(PAIR3));
        
        final Map<Integer, MaintainerTask> map1 = maintainer.maintainerMapByPair.get(PAIR1);
        assertEquals(2, map1.size());
        assertTrue("" + map1, map1.containsValue(new ENTITY1_v0_v1()));
        assertTrue("" + map1, map1.containsValue(new ENTITY1_v1_v2()));
        
        final Map<Integer, MaintainerTask> map2 = maintainer.maintainerMapByPair.get(PAIR2);
        assertEquals(2, map2.size());
        assertTrue("" + map2, map2.containsValue(new OLD_ENTITY2_v0_v1()));
        assertTrue("" + map2, map2.containsValue(new OLD_ENTITY2_v1_v2()));
        
        final Map<Integer, MaintainerTask> map3 = maintainer.maintainerMapByPair.get(PAIR3);
        assertEquals(1, map3.size());
        assertTrue("" + map3, map3.containsValue(new ENTITY2_v0_v1()));
      }

    @Test
    public void testInitialVersions()   
      throws Exception
      {
        logger.info("******** testInitialVersions");
        maintainer.openConnection();
        maintainer.createTableVersionTable();
        assertEquals(0, maintainer.getVersion("ENTITY1"));
        assertEquals(0, maintainer.getVersion("OLD_ENTITY2"));
        assertEquals(0, maintainer.getVersion("ENTITY2"));
      }

    @Test
    public void testRun()   
      throws Exception
      {        
        logger.info("******** testRun");
        final EntityAndTablePair PAIR1 = new EntityAndTablePair(Entity1.class, "ENTITY1");
        final EntityAndTablePair PAIR2 = new EntityAndTablePair(Entity2.class, "OLD_ENTITY2");
        final Map<Integer, MaintainerTask> map1 = maintainer.maintainerMapByPair.get(PAIR1);
        final Map<Integer, MaintainerTask> map2 = maintainer.maintainerMapByPair.get(PAIR2);
        
        final MaintainerTask m_e1_v0_1 = map1.get(0);
        final MaintainerTask m_e1_v1_2 = map1.get(1);
        final MaintainerTask m_e2_v0_1 = map2.get(0);
        final MaintainerTask m_e2_v1_2 = map2.get(1);
        
        maintainer.run();

//        assertArrayEquals(Arrays.asList("OLD_ID", "INT1", "OLD_INT2", "STRING1", "OLD_STRING2"), TestUtil.getOldColumns(m_e1_v0_1));
//        assertArrayEquals(Arrays.asList("OLD_ID", "INT1", "OLD_INT2", "STRING1", "OLD_STRING2"), TestUtil.getNewColumns(m_e1_v0_1));
        
        maintainer.openConnection(); // for making getVersion() work
        assertEquals(2, maintainer.getVersion(Entity1.class.getAnnotation(Table.class).name()));
        assertEquals(2, maintainer.getVersion(Entity2.class.getAnnotation(Table.class).name()));
        
// TODO        assertFalse(maintainer.existsTable(TestUtil.findManipulator(m_e1_v0_1).tempTableName));
// TODO       assertFalse(maintainer.existsTable(TestUtil.findManipulator(m_e2_v0_1).tempTableName));

        
        // TODO: assert tables structure is ok
        // TODO: assert data has been copied
      }
  }