/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: MaintainerTestSupport.java,v 06a3150da44c 2010/04/05 02:15:33 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.persistence.maintenance.impl;

import javax.annotation.Nonnull;
import java.util.Properties;
import java.io.File;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.Statement;
import it.tidalwave.netbeans.persistence.Persistence;
import it.tidalwave.netbeans.persistence.maintenance.impl.mock.TestPersistenceConfigurator;
import it.tidalwave.netbeans.persistence.spi.PersistenceConfigurator;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.netbeans.util.test.TestLoggerSetup;
import it.tidalwave.util.logging.Logger;
import java.sql.Driver;
import org.apache.commons.io.FileUtils;
import org.junit.After;
import org.junit.Before;
import org.junit.BeforeClass;
import static org.junit.Assert.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: MaintainerTestSupport.java,v 06a3150da44c 2010/04/05 02:15:33 fabrizio $
 *
 **********************************************************************************************************************/
public class MaintainerTestSupport
  {
    private static final String CLASS = MaintainerTestSupport.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);
    
    protected static Persistence persistence;
    
    /***************************************************************************
     *
     **************************************************************************/
    @BeforeClass
    public static void setupLogger()
      {
        TestLoggerSetup.setupLogging(MaintainerTestSupport.class);
      }
    
    /***************************************************************************
     *
     **************************************************************************/
    @Before
    public void setupDatabase()
      throws Exception
      {
        try
          {
            final File tempFolder = new File("target/test-artifacts").getAbsoluteFile();
            logger.info("***** Scratching and recreating working directory: %s ...", tempFolder);
            FileUtils.deleteDirectory(tempFolder);
            assertTrue(tempFolder.mkdirs());
            final String dbPath = new File(tempFolder, "TestDatabase").getAbsolutePath();
            ((TestPersistenceConfigurator)Locator.find(PersistenceConfigurator.class)).setDatabaseFolder(dbPath);

            final File workspaceFolder = new File(tempFolder, "TestPersistanceMaintainer");
            System.setProperty("it.tidalwave.netbeans.workspacemanager.workspace", workspaceFolder.getAbsolutePath());
            System.setProperty("it.tidalwave.netbeans.workspacemanager.scratchWorkspace", "true");

            createOriginalTables(); // before booting Persistence!
            shutDownDatabase(); // because we're simulating a pre-existing, closed database
            logger.info("***** Booting Persistence...");
            persistence = Locator.find(Persistence.class);
          }
        catch (Exception e)
          {
            logger.severe("Coulndn't setup the test database: %s", e);
            logger.throwing(CLASS, "setupDatabase()", e);
            throw e;
          }
      }

    @After
    public void shutDownDatabase()
      {
        logger.info("***** Shutting down test database...");
        Locator.find(PersistenceConfigurator.class).shutdown();
      }

    protected static void createOriginalTables()
      throws Exception
      {
        logger.info("***** Creating test database...");
        final Connection connection = createConnection();
        logger.info("***** database URL: %s", connection.getMetaData().getURL());
        final Statement statement = connection.createStatement();
        statement.executeUpdate("CREATE TABLE ENTITY1 (OLD_ID VARCHAR(10), INT1 INT, OLD_INT2 INT, STRING1 VARCHAR(10), OLD_STRING2 VARCHAR(10))");
        
        for (int i = 0; i < 10; i++)
          {
            String sql = String.format("INSERT INTO ENTITY1(OLD_ID, INT1, OLD_INT2, STRING1, OLD_STRING2) VALUES('id-%1$d', %1$d, %1$d, 's1.%1$d', 's2.%1$d')", i);
            statement.executeUpdate(sql);
          }
        
        statement.executeUpdate("CREATE TABLE OLD_ENTITY2 (OID VARCHAR(10), INT1 INT, STRING1 VARCHAR(10), OLD VARCHAR(10))");
        
        for (int i = 0; i < 10; i++)
          {
            String sql = String.format("INSERT INTO OLD_ENTITY2(OID, INT1, STRING1, OLD) VALUES('id-%1$d', %1$d, 's1.%1$d', 's2.%1$d')", i);
            statement.executeUpdate(sql);
          }
        
        statement.close();
        connection.close();
      }

    @Nonnull
    protected static Connection createConnection()
      throws Exception
      {
        // Don't lookup Persistence or you'd initialize it.
        final Properties properties = Locator.find(PersistenceConfigurator.class).initialize();
        final String dbURL = properties.getProperty("connection.url");
        assertNotNull("Can't find connection.url", dbURL);
        final String driverClass = properties.getProperty("connection.driver_class");
        final Driver driver = (Driver)Thread.currentThread().getContextClassLoader().loadClass(driverClass).newInstance();
        return driver.connect(dbURL, properties);
      }
  }
