/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: PersistenceImplTest.java,v 5fae2567b1ad 2010/03/22 19:21:14 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.persistence.impl;

import java.util.HashSet;
import java.util.Properties;
import java.util.Set;
import java.io.Serializable;
import java.sql.DatabaseMetaData;
import java.sql.SQLException;
import javax.persistence.EntityManagerFactory;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.netbeans.persistence.Persistence;
import it.tidalwave.netbeans.persistence.TxTask;
import it.tidalwave.netbeans.persistence.spi.PersistenceConfigurator;
import it.tidalwave.netbeans.persistence.mock.MockEntity;
import it.tidalwave.netbeans.persistence.mock.TestPersistenceConfigurator;
import it.tidalwave.netbeans.util.test.TestLoggerSetup;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: PersistenceImplTest.java,v 5fae2567b1ad 2010/03/22 19:21:14 fabrizio $
 *
 **********************************************************************************************************************/
public class PersistenceImplTest
  {
    private static final String CLASS = PersistenceImplTest.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);
    
    private PersistenceImpl fixture;

    @BeforeClass
    public static void setupLogger()
      {
        TestLoggerSetup.setupLogging(PersistenceImplTest.class);
      }

    @Before
    public void setupFixture()
      {
        fixture = new PersistenceImpl();
      }

    @Test
    public void mustBeRegisteredInMETA_INF()
      {
        assertTrue(Locator.find(Persistence.class) instanceof PersistenceImpl);
      }

    @Test
    public void testShutdown()
      {
        logger.info("**************** shutdown");
        TestPersistenceConfigurator tpc = (TestPersistenceConfigurator)Locator.find(PersistenceConfigurator.class);
        assertFalse(tpc.isClosed());
        fixture.shutdown();
        assertTrue(tpc.isClosed());
      }

    @Test
    public void testGetProperties()
      {
        logger.info("**************** getProperties");
        Properties properties = fixture.getProperties();
        assertEquals(7, properties.size());
        // TODO: assert values
      }

    @Test
    public void testGetEntityManagerFactory()
      {
        logger.info("**************** getEntityManagerFactory");
        final EntityManagerFactory emf = fixture.getEntityManagerFactory();
        assertNotNull(emf);
        assertTrue(emf.isOpen());
      }

    @Test
    public void testReset()
      {
        logger.info("**************** reset");
        final EntityManagerFactory emf1 = fixture.getEntityManagerFactory();
        fixture.reset();
        final EntityManagerFactory emf2 = fixture.getEntityManagerFactory();
        assertNotNull(emf2);
        assertNotSame(emf1, emf2);
        assertFalse(emf1.isOpen());
        assertTrue(emf2.isOpen());
      }

    @Test
    public void testCreateIndexes()
      {
        logger.info("**************** createIndexes");
        // repeated multiple times, shouldn't fail the next times
        fixture.createIndexes(MockEntity.class, "field1", "field2");
        fixture.createIndexes(MockEntity.class, "field1", "field2");

        fixture.reset();
        fixture.createIndexes(MockEntity.class, "field1", "field2");
      }

    @Test
    public void testTableExists()
      {
        logger.info("**************** tableExists");
        assertTrue(fixture.tableExists("MOCKENTITY"));
        assertFalse(fixture.tableExists("FOO"));
        assertFalse(fixture.tableExists("BAR"));
      }

    @Test
    public void testCreateUniqueId()
      {
        logger.info("**************** createUniqueId");
        final Set<Serializable> generatedIds = new HashSet<Serializable>();

        for (int i = 0; i < 100000; i++)
          {
            final Serializable id = fixture.createUniqueId();
            assertFalse(generatedIds.contains(id));
            generatedIds.add(id);
            assertEquals(i + 1, generatedIds.size());
          }
      }

    @Test
    public void testGetDatabaseMetaData() 
      throws Exception
      {
        TxTask.execute(SQLException.class, new TxTask<Void, SQLException>()
          {
            @Override
            public Void run()
              throws SQLException
              {
                logger.info("**************** getDatabaseMetaData");
                final DatabaseMetaData dbmd = fixture.getDatabaseMetaData();
                assertNotNull(dbmd);
                return null;
              }
          });
      }
//
//    @Test
//    public void testDataSourceIsRegistered()
//      throws Exception
//      {
//        final InitialContext context = new InitialContext();
//        final Object lookup = context.lookup(Persistence.DATASOURCE_NAME);
//        assertNotNull(lookup);
//        assertTrue(lookup instanceof DataSourceImpl);
//      }
  }