/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: Actions.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.util.actions;

import javax.annotation.Nonnull;
import java.awt.event.ActionListener;
import javax.swing.Action;
import org.openide.util.Parameters;
import org.openide.filesystems.FileObject;
import it.tidalwave.netbeans.util.actions.impl.EnhancedIconAction;

/***********************************************************************************************************************
 *
 * This class contains a few utility static methods for creating {@Action} instances.
 * 
 * @author  Fabrizio Giudici
 * @version $Id: Actions.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public final class Actions
  {
    /*******************************************************************************************************************
     *
     * Creates an {@link Action} which is always enabled and delegates to an existing {@link ActionListener}. The
     * delegate is described by the given {@link FileObject} by means of the following attributes:
     *
     * <ul>
     * <li>delegate: the delegate instance</li>
     * <li>displayName: the display name</li>
     * <li>iconBase: the path to the icon</li>
     * <li>noIconInMenu: if true, this attribute disables any icon</li>
     * </ul>
     *
     ******************************************************************************************************************/
    @Nonnull
    public static Action alwaysEnabled (final @Nonnull FileObject fileObject)
      {
        Parameters.notNull("fileObject", fileObject);
        final Object noIconInMenu = fileObject.getAttribute("noIconInMenu");
        final Action delegate =
            org.openide.awt.Actions.alwaysEnabled((ActionListener)fileObject.getAttribute("delegate"),
                                                  (String)fileObject.getAttribute("displayName"),
                                                  (String)fileObject.getAttribute("iconBase"),
                                                  ((noIconInMenu != null) ? (Boolean)noIconInMenu : Boolean.FALSE));
        return new EnhancedIconAction(delegate);
      }
  }
