/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: EnhancedCapabilityAction.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.util.actions;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import java.util.MissingResourceException;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.CookieAction;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: EnhancedCapabilityAction.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public abstract class EnhancedCapabilityAction extends CookieAction // WithResizableIcon
  {
    @Nonnull
    private String name;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    protected EnhancedCapabilityAction()
      {
        name = getClass().getName();
        int i = name.lastIndexOf('.');

        if (i >= 0)
          {
            name = name.substring(i + 1);
          }
      }

    /*******************************************************************************************************************
     *
     * <code>GenericCookieAction</code> provides a convenient default implementation of <code>getName()</code>. If the 
     * resource name for your action is just <code>CTL_</code><i>action_name</i>, you needn't to override this method.
     *
     * @return  the name of this action
     *
     ******************************************************************************************************************/
    @Override
    @Nonnull
    public String getName()
      {
        return NbBundle.getMessage(getClass(), "CTL_" + name);
      }

    /*******************************************************************************************************************
     *
     * <code>GenericCookieAction</code> provides a convenient default implementation of <code>iconResource()</code>. If 
     * the resource name for your action is just <code>ICON_</code><i>action_name</i>, you needn't to override this method.
     *
     * @return  the name of this action
     *
     ******************************************************************************************************************/
    @Override
    @CheckForNull
    protected String iconResource()
      {
        try
          {
            String s = NbBundle.getMessage(getClass(), "ICON_" + name).trim();
            return (s.indexOf(".svg") >= 0) ? null : s;
          }
        catch (MissingResourceException e)
          {
            putValue("noIconInMenu", Boolean.TRUE);
            return super.iconResource();
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @CheckForNull
    public String svgIconResource()
      {
        try
          {
            String s = NbBundle.getMessage(getClass(), "ICON_" + name).trim();
            return (s.indexOf(".svg") < 0) ? null : s;
          }
        catch (MissingResourceException e)
          {
            putValue("noIconInMenu", Boolean.TRUE);
            return super.iconResource();
          }
      }

    /*******************************************************************************************************************
     *
     * Convenience method for retrieving a bundled message. The resource name used is <code>FN_</code>
     * <i>action_name.resourceName</i>.
     *
     * @param   resourceName  the resource name
     * @oaram   parameters    optional parameters
     * @return                the bundled message
     *
     ******************************************************************************************************************/
    @Nonnull
    protected String getBundleMessage (final @Nonnull String resourceName, final @Nonnull Object ... parameters)
      {
        return NbBundle.getMessage(getClass(), "FN_" + name + "." + resourceName, parameters);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    @Nonnull
    public HelpCtx getHelpCtx()
      {
        return HelpCtx.DEFAULT_HELP;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override
    final protected boolean asynchronous()
      {
        return false;
      }
  }
