/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: SelectFileAction.java,v e168a2226107 2010/03/28 20:51:39 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.util.actions;

import it.tidalwave.netbeans.dialog.DialogRunner;
import it.tidalwave.netbeans.util.actions.spi.DefaultSelectFileActionDelegate;
import it.tidalwave.netbeans.util.actions.spi.SelectFileActionDelegate;
import java.util.logging.Logger;
import java.io.File;
import java.awt.Component;
import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import javax.swing.JFileChooser;
import org.openide.util.Utilities;
import org.openide.windows.WindowManager;
import org.openide.util.Lookup;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: SelectFileAction.java,v e168a2226107 2010/03/28 20:51:39 fabrizio $
 *
 **********************************************************************************************************************/
public abstract class SelectFileAction extends AbstractAction
  {
    private static final String CLASS = SelectFileAction.class.getName();
    
    private static final Logger logger = Logger.getLogger(CLASS);
    
    private Component component;
    
    private String title;
    
    private int type;
    
    private File initialFolder = new File(System.getProperty("user.home"));
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    public SelectFileAction (String title, int type) 
      {
        this(null, null, title, type);
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    public SelectFileAction (Component component, String title, int type) 
      {
        this(component, null, title, type);
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    public SelectFileAction (Component component, String name, String title, int type) 
      {
        super(name);
        this.component = component;
        this.title = title;
        this.type = type;
        
        if (this.component == null)
          {
            this.component = WindowManager.getDefault().getMainWindow();   
          }
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    public static void runAction (SelectFileAction action)
      {
        action.actionPerformed(null);
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    @Override
    final public void actionPerformed (ActionEvent event) 
      {
        try 
          {
            final JFileChooser fileChooser = new JFileChooser(initialFolder);
            fileChooser.setDialogTitle(title);
            fileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
            fileChooser.setFileSelectionMode(type);
            fileChooser.setMultiSelectionEnabled(false);
            
            SelectFileActionDelegate delegate = new DefaultSelectFileActionDelegate();

            if ((Utilities.getOperatingSystem() == Utilities.OS_MAC) 
                && (DialogRunner.getCurrentModalDialog() == null)) // FIXME: doesn't work with modal dialogs opened
              {
                for (final SelectFileActionDelegate specialDelegate : Lookup.getDefault().lookupAll(SelectFileActionDelegate.class))
                  {
                    if (delegate.getOperatingSystem() == Utilities.getOperatingSystem())
                      {
                        delegate = specialDelegate;
                        break;
                      }
                  }
              }

            delegate.runOnConfirmation(fileChooser, component, new SelectFileActionDelegate.Notifier()
              {
                @Override
                public void notify (final File file)
                  {
                    notifyFileSelected(file);
                  }
              });
          } 
        catch (Throwable t) 
          {
            logger.throwing(CLASS, "actionPerformed()", t);
            // TODO: show error to user
          }
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    final protected boolean asynchronous() 
      {
        return false;
      }  

    /***************************************************************************
     *
     *
     **************************************************************************/
    public abstract void notifyFileSelected (File file);
  }
