/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: EventDeliverRole.java,v 4e8a394a80a9 2010/03/28 20:51:11 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.loaders.role;

import javax.annotation.Nonnull;
import org.openide.util.Lookup;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.netbeans.loaders.EnhancedMultiDataObject;
import it.tidalwave.netbeans.loaders.event.DataObjectEvent;
import it.tidalwave.netbeans.loaders.event.DataObjectListener;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public final class EventDeliverRole implements LifeCycleRole
  {
    private static final String CLASS = EventDeliverRole.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    public void dataObjectCreated (final @Nonnull EnhancedMultiDataObject dataObject)
      {
        logger.fine("dataObjectCreated(%s)", dataObject);
        final DataObjectEvent event = new DataObjectEvent(this, dataObject);

        for (final DataObjectListener listener : Lookup.getDefault().lookupAll(DataObjectListener.class))
          {
            try
              {
                listener.notifyDataObjectCreated(event);
              }
            catch (Throwable t)
              {
                logger.throwing(CLASS, "", t);
              }
          }
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    public void dataObjectRemoved (final @Nonnull EnhancedMultiDataObject dataObject)
      {
        logger.fine("dataObjectRemoved(%s)", dataObject);
        final DataObjectEvent event = new DataObjectEvent(this, dataObject);

        for (final DataObjectListener listener : Lookup.getDefault().lookupAll(DataObjectListener.class))
          {
            try
              {
                listener.notifyDataObjectRemoved(event);
              }
            catch (Throwable t)
              {
                logger.throwing(CLASS, "", t);
              }
          }
      }
  }
