/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: EnhancedDataObjectTest.java,v 4e8a394a80a9 2010/03/28 20:51:11 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.loaders;

import it.tidalwave.netbeans.loaders.role.LifeCycleRole;
import it.tidalwave.netbeans.util.test.TestLoggerSetup;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import javax.annotation.Nonnull;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.MultiFileLoader;
import org.openide.loaders.UniFileLoader;
import static org.mockito.Mockito.*;
import org.openide.util.Lookup;
import org.openide.util.lookup.Lookups;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public class EnhancedDataObjectTest
  {
    public static class EnhancedDataObjectImpl extends EnhancedMultiDataObject
      {
        public EnhancedDataObjectImpl (final @Nonnull FileObject fileObject,
                                       final @Nonnull MultiFileLoader dataLoader)
          throws DataObjectExistsException
          {
            super(fileObject, dataLoader);
          }

        @Override @Nonnull
        Lookup createLookup()
          {
            return Lookups.fixed(capabilities.toArray());
          }
      }

    private FileSystem fileSystem;

    private FileObject fileObject;

    private UniFileLoader dataLoader;

    private static List<Object> capabilities;

    @BeforeClass
    public static void setupLogging()
      {
        TestLoggerSetup.setupLogging(EnhancedDataObjectTest.class);
      }

    @Before
    public void setup()
      throws IOException
      {
        fileSystem = FileUtil.createMemoryFileSystem();
//        assertThat // (fileSystem, is(not(nullValue(FileSystem.class))));
        fileObject = fileSystem.getRoot().createData("foo.bar");
//        assertThat(fileObject, is(not(nullValue())));

        dataLoader = new UniFileLoader(EnhancedDataObjectImpl.class.getName())
          {
            @Override @Nonnull
            protected MultiDataObject createMultiObject (final FileObject fileObject)
              throws DataObjectExistsException, IOException
              {
                return new EnhancedDataObjectImpl(fileObject, this);
              }
          };

        dataLoader.getExtensions().addExtension("bar");
        capabilities = new ArrayList<Object>();
      }

    @Test
    public void lifeCycleRoleMustBeInvokedAtCreation()
      throws Exception
      {
        final LifeCycleRole lifeCycleRole = mock(LifeCycleRole.class);
        capabilities.add(lifeCycleRole);
        final DataObject dataObject = dataLoader.findDataObject(fileObject, Collections.emptySet());
        verify(lifeCycleRole).dataObjectCreated(same((EnhancedMultiDataObject)dataObject));
        verifyNoMoreInteractions(lifeCycleRole);
      }

    @Test
    public void lifeCycleRoleMustBeInvokedAtDisposal()
      throws Exception
      {
        final LifeCycleRole lifeCycleRole = mock(LifeCycleRole.class);
        capabilities.add(lifeCycleRole);
        final DataObject dataObject = dataLoader.findDataObject(fileObject, Collections.emptySet());
        reset(lifeCycleRole);
        dataObject.delete();
        verify(lifeCycleRole).dataObjectRemoved(same((EnhancedMultiDataObject)dataObject));
        verifyNoMoreInteractions(lifeCycleRole);
      }
  }
