/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: ExplorerSelectionStrategy.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.explorer.role;

import javax.annotation.Nonnull;
import javax.annotation.PostConstruct;
import javax.inject.Inject;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import org.openide.util.Lookup;
import org.openide.util.Parameters;
import org.openide.util.NbPreferences;
import org.openide.nodes.Node;
import org.openide.explorer.ExplorerManager;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: ExplorerSelectionStrategy.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public abstract class ExplorerSelectionStrategy<T>
  {
    public final static String AUTO_SELECTION = "autoSelection/3";

    @Inject
    private Lookup lookup;

    @Nonnull
    private final Class<T> selectedObjectClass;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public static void setAutoSelectionEnabled (final boolean autoSelection)
      {
        NbPreferences.forModule(ExplorerSelectionStrategy.class).putBoolean(AUTO_SELECTION, autoSelection);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public static boolean isAutoSelectionEnabled()
      {
         return NbPreferences.forModule(ExplorerSelectionStrategy.class).getBoolean(AUTO_SELECTION, true);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private final PropertyChangeListener nodeSelectionListener = new PropertyChangeListener()
      {
        @Override
        public void propertyChange (final PropertyChangeEvent event)
          {
            if ("selectedNodes".equals(event.getPropertyName()))
              {
                final Node[] nodes = (Node[])event.getNewValue();

                if ((nodes.length > 0) && isAutoSelectionEnabled())
                  {
                    final Collection<T> objects = getSelectedObjects(nodes);

                    if (!objects.isEmpty())
                      {
                        notifySelection(objects);
                      }
                  }
              }
          }
      };

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public ExplorerSelectionStrategy (@Nonnull final Class<T> selectedObjectClass)
      {
        Parameters.notNull("selectedObjectClass", selectedObjectClass);
        this.selectedObjectClass = selectedObjectClass;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @PostConstruct
    @SuppressWarnings("PMD.UnusedPrivateMethod") @edu.umd.cs.findbugs.annotations.SuppressWarnings("UPM_UNCALLED_PRIVATE_METHOD")
    private void initialize()
      {
        final ExplorerManager explorerManager = lookup.lookup(ExplorerManager.class);

        if (explorerManager == null)
          {
            throw new IllegalArgumentException("null explorerManager");
          }
        
        explorerManager.addPropertyChangeListener(nodeSelectionListener);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    protected abstract void notifySelection (@Nonnull final Collection<T> selectedObjects);

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private List<T> getSelectedObjects (@Nonnull final Node[] nodes)
      {
        final List<T> objects = new ArrayList<T>();

        for (final Node node : nodes)
          {
            final T object = node.getLookup().lookup(selectedObjectClass);

            if (object != null)
              {
                objects.add(object);
              }
          }

        return objects;
      }
  }
