/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: EnhancedBeanTreeView.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.explorer.view;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import javax.swing.UIManager;
import javax.swing.tree.TreeCellRenderer;
import org.openide.explorer.view.BeanTreeView;
import it.tidalwave.netbeans.explorer.view.impl.OldPatchedNodeRenderer;

/*******************************************************************************
 *
 * A customisation of BeanTreeView which exposes methods for:
 * <ul>
 * <li>changing the colors;</li>
 * <li>setting a custom cell renderer;</li>
 * </ul>
 *
 * @author  Fabrizio Giudici
 * @version $Id: EnhancedBeanTreeView.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 ******************************************************************************/
public class EnhancedBeanTreeView extends BeanTreeView
  {
//    private TreeCellRenderer treeCellRenderer;
    private boolean treeInitialized = false;
    private boolean rendererPatched = false;

    /*******************************************************************************
     *
     ******************************************************************************/
    @Override
    public void setFont (final Font font)
      {
        super.setFont(font);

        if (tree != null)
          {
            tree.setFont(font);
          }
      }

    /*******************************************************************************
     *
     ******************************************************************************/
    @Override
    public void setBackground (final Color color)
      {
        super.setBackground(color);

        if (tree != null)
          {
            tree.setBackground(color);
            tree.putClientProperty("JTree.lineStyle", "None"); // FIXME: move elsewhere
          }
      }

    /*******************************************************************************
     *
     ******************************************************************************/
    @Override
    public void setForeground (final Color color)
      {
        super.setForeground(color);

        if (tree != null)
          {
            tree.setForeground(color);
          }
      }

    /*******************************************************************************
     *
     ******************************************************************************/
    @Override
    public void setOpaque (final boolean opaque)
      {
        super.setOpaque(opaque);

        if (tree != null)
          {
            tree.setOpaque(opaque);
          }
      }

    /*******************************************************************************
     *
     ******************************************************************************/
    public void setCellRenderer (final TreeCellRenderer treeCellRenderer)
      {
//        this.treeCellRenderer = treeCellRenderer;

        if (tree != null)
          {
            tree.setCellRenderer(treeCellRenderer);
          }
      }

    /*******************************************************************************
     *
     *
     ******************************************************************************/
    public TreeCellRenderer getCellRenderer()
      {
        patchRenderer(); // so callers would get the real renderer, eventually patched
        return tree.getCellRenderer();
      }

    /*******************************************************************************
     *
     *
     ******************************************************************************/
    @Override
    public void paint (final Graphics g)
      {
        initializeTree2();
        super.paint(g);
      }

    /*******************************************************************************
     *
     *
     ******************************************************************************/
    private void initializeTree2()
      {
        if (!treeInitialized)
          {
            tree.setOpaque(isOpaque());
            tree.setBackground(getBackground());
            tree.setForeground(getForeground());
            treeInitialized = true;
          }

        patchRenderer();
      }

    /*******************************************************************************
     *
     *
     ******************************************************************************/
    private void patchRenderer()
      {
        if (Boolean.getBoolean("nb.useSwingHtmlRendering") && !rendererPatched)
          {
            rendererPatched = true;
            final OldPatchedNodeRenderer nodeRenderer = new OldPatchedNodeRenderer(tree.getCellRenderer());
            nodeRenderer.setBackgroundSelectionColor(getColorProperty("Tree.selectionBackground"));
            nodeRenderer.setBackgroundNonSelectionColor(tree.getBackground());
            nodeRenderer.setTextSelectionColor(getColorProperty("Tree.selectionForeground"));
            nodeRenderer.setTextNonSelectionColor(tree.getForeground());
            tree.setCellRenderer(nodeRenderer);
          }
      }

    /*******************************************************************************
     *
     *
     ******************************************************************************/
    private Color getColorProperty (final String propertyName)
      {
        final Object property = getClientProperty(propertyName);

        if (property != null)
          {
            return (Color)property;
          }

        else
          {
            return UIManager.getColor(propertyName);
          }
      }
  }
