/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: OldPatchedNodeRenderer.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.explorer.view.impl;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.tree.TreeCellRenderer;

/*******************************************************************************
 *
 * When nb.useSwingHtmlRendering=true, that is HtmlLabelUI is not used, the
 * selection colors are not ok. This class fixes them.
 * Also, TreeTable does show properties in a right column, but I couldn't customize it.
 * See http://openbluesky.dev.java.net/issues/browse/BM-223
 * This class provides a simple way to renderer a text property near the node.
 *
 * FIXME: split the property rendering to a separate component.
 *
 * @author  Fabrizio Giudici
 * @version $Id: OldPatchedNodeRenderer.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 ******************************************************************************/
// TODO: must be dropped since we don't need any longer NodePropertyProvider
public class OldPatchedNodeRenderer extends JPanel implements TreeCellRenderer
  {
    private TreeCellRenderer peer;
    private Component component;
//    private JTree tree;
//    private String text;
//    private Font smallFont;
    private Color backgroundNonSelectionColor = Color.BLACK;
    private Color backgroundSelectionColor = Color.WHITE;
    private Color textNonSelectionColor = Color.WHITE;
    private Color textSelectionColor = Color.BLACK;
//    private NodePropertyProvider nodePropertyProvider;

    /*******************************************************************************
     *
     ******************************************************************************/
    public OldPatchedNodeRenderer (final TreeCellRenderer peer)
      {
        this.peer = peer;
        setLayout(new BorderLayout());
        setOpaque(false);
      }

    /*******************************************************************************
     *
     ******************************************************************************/
    public void setBackgroundSelectionColor (final Color backgroundSelectionColor)
      {
        this.backgroundSelectionColor = backgroundSelectionColor;
      }

    /*******************************************************************************
     *
     ******************************************************************************/
    public Color getBackgroundSelectionColor()
      {
        return backgroundSelectionColor;
      }

    /*******************************************************************************
     *
     ******************************************************************************/
    public void setBackgroundNonSelectionColor (final Color backgroundNonSelectionColor)
      {
        this.backgroundNonSelectionColor = backgroundNonSelectionColor;
      }

    /*******************************************************************************
     *
     ******************************************************************************/
    public void setTextNonSelectionColor (final Color textNonSelectionColor)
      {
        this.textNonSelectionColor = textNonSelectionColor;
      }

    /*******************************************************************************
     *
     ******************************************************************************/
    public void setTextSelectionColor (final Color textSelectionColor)
      {
        this.textSelectionColor = textSelectionColor;
      }

    /*******************************************************************************
     *
     ******************************************************************************/
    public Color getTextSelectionColor()
      {
        return textSelectionColor;
      }

    /*******************************************************************************
     *
     ******************************************************************************/
//    public void setNodePropertyProvider (final NodePropertyProvider nodePropertyProvider)
//      {
//        this.nodePropertyProvider = nodePropertyProvider;
//      }

    /*******************************************************************************
     *
     ******************************************************************************/
//    @Override
//    public void paint (final Graphics g)
//      {
//        super.paint(g);
//
//        if (nodePropertyProvider != null)
//          {
//            g.setColor(component.getForeground());
//            g.setFont(smallFont);
//            final Rectangle saveClip = g.getClipBounds();
//            g.setClip(saveClip.x, saveClip.y, tree.getWidth(), saveClip.height);
//            final FontMetrics fontMetrics = g.getFontMetrics();
//            final FontMetrics fontMetrics2 = g.getFontMetrics(component.getFont());
//
//            int x = tree.getWidth() - fontMetrics.stringWidth(text) - getX() - 4;
//            final int y = getHeight() - (getHeight() - fontMetrics2.getHeight()) / 2 - fontMetrics2.getDescent();
//            g.drawString(text, x, y);
//            g.setClip(saveClip);
//          }
//      }

    /*******************************************************************************
     *
     ******************************************************************************/
    public Component getTreeCellRendererComponent (final JTree tree,
                                                   final Object object,
                                                   final boolean selected,
                                                   final boolean expanded,
                                                   final boolean leaf,
                                                   final int row,
                                                   final boolean hasFocus)
      {
        final Component newComponent = peer.getTreeCellRendererComponent(tree, object, selected, expanded, leaf, row, hasFocus);

        if (newComponent != component)
          {
            if (component != null)
              {
                remove(component);
              }

            add(component = newComponent, BorderLayout.CENTER);
            final Font font = component.getFont();
//            smallFont = font.deriveFont(0.8f * font.getSize2D()); // FIXME: cache it?
          }

        component.setBackground(selected ? backgroundSelectionColor : backgroundNonSelectionColor);
        component.setForeground(selected ? textSelectionColor : textNonSelectionColor);
//        this.tree = tree;

//        if (nodePropertyProvider != null)
//          {
//            final Node node = Visualizer.findNode(object);
//            text = nodePropertyProvider.getText(node);
//          }

        return this;
      }

    /*******************************************************************************
     *
     ******************************************************************************/
    @Override
    public Dimension getSize()
      {
        if (component == null)
          {
            return super.getSize();
          }

        return component.getSize();
      }

    /*******************************************************************************
     *
     ******************************************************************************/
    @Override
    public Dimension getPreferredSize()
      {
        if (component == null)
          {
            return super.getPreferredSize();
          }

        return component.getPreferredSize();
      }
  }

