/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: ExplorerPanelTest.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.explorer;

import javax.swing.JLabel;
import javax.swing.JComponent;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import it.tidalwave.netbeans.nodes.role.RootNodeProvider;
import org.netbeans.platformx.inject.api.Injector;
import org.junit.Before;
import org.junit.Test;
import org.mockito.InOrder;
import org.openide.explorer.ExplorerManager;
import org.openide.util.lookup.Lookups;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;
import static org.hamcrest.CoreMatchers.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: ExplorerPanelTest.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public class ExplorerPanelTest
  {
    private Node rootNode;

    private boolean waitRootNodeDelivery;

    private final RootNodeProvider rootNodeProvider = new RootNodeProvider()
      {
        @Override
        public Node getRootNode()
          {
            while (waitRootNodeDelivery)
              {
                try
                  {
                    Thread.sleep(50);
                  }
                catch (InterruptedException e)
                  {
                    e.printStackTrace();
                  }
              }
            
            return rootNode;
          }
      };

    @Before
    public void setUp()
      {
        rootNode = new AbstractNode(Children.LEAF);
        rootNode.setDisplayName("data node");
        waitRootNodeDelivery = true;
      }

    @Test
    public void shouldApplyTheProperSequenceWhenLoadingData()
      throws InterruptedException
      {
        final ExplorerPanel.TransitionController transitionController = mock(ExplorerPanel.TransitionController.class);
        final ExplorerManager explorerManager = new ExplorerManager(); // final class, can't be mocked
        doReturn(new JLabel()).when(transitionController).getComponent();
        final JComponent view = mock(JComponent.class);

        final ExplorerPanel explorerPanel = new ExplorerPanel();
        Injector.getDefault().inject(explorerPanel, Lookups.fixed(transitionController, explorerManager, view));
        
        final InOrder inOrder = inOrder(transitionController);

        final int uc = explorerPanel.updateCounter;
        
        explorerPanel.setRootNodeProvider(rootNodeProvider);
        inOrder.verify(transitionController).lock(eq("Loading..."));
        // TODO: assert EDT not blocked

        waitRootNodeDelivery = false;

        while (uc == explorerPanel.updateCounter)
          {
            Thread.sleep(50);
          }

        assertNotNull(rootNode);
        assertThat(explorerPanel.getExplorerManager().getRootContext(), is(sameInstance(rootNode)));
        inOrder.verify(transitionController).prepareUnlock();
        inOrder.verify(transitionController).unlock();
      }
  }