/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: ClassLoaderDecorator.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.jpa;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import java.util.Enumeration;
import java.io.IOException;
import java.net.URL;

/***********************************************************************************************************************
 *
 * Decorates a {@link ClassLoader} so that the method {@link #getResources(String)} delegates to a
 * {@link MultiPersistenceXmlManager} for providing a resource named {@code META-INF/persistence.xml}.
 * 
 * @author Fabrizio Giudici
 * @version $Id: ClassLoaderDecorator.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public class ClassLoaderDecorator extends ClassLoader 
  {
    private static final String PERSISTENCE_XML = "META-INF/persistence.xml";

    private final MultiPersistenceXMLManager multiPersistenceXmlManager;
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public ClassLoaderDecorator (final @Nonnull ClassLoader parent,
                                 final @Nonnull MultiPersistenceXMLManager multiPersistenceXmlManager)
      {
        super(parent);
        this.multiPersistenceXmlManager = multiPersistenceXmlManager;
      }
    
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public Enumeration<URL> getResources (final @Nonnull String name)
      throws IOException 
      {
        if (PERSISTENCE_XML.equals(name))
          {
            return new Enumeration<URL>() 
              {
                URL url = multiPersistenceXmlManager.getPersistenceXmlUrl();

                @Override
                public boolean hasMoreElements() 
                  {
                    return url != null;
                  }

                @Override @CheckForNull
                public URL nextElement() 
                  {
                    final URL url2 = url;
                    url = null;
                    return url2;
                  }
              };
          }
        
        return super.getResources(name);
      }
  }
