/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: PersistenceProviderDecorator.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.jpa;

import javax.annotation.Nonnull;
import java.util.Map;
import javax.persistence.EntityManagerFactory;
import javax.persistence.spi.PersistenceProvider;
import javax.persistence.spi.PersistenceUnitInfo;

/***********************************************************************************************************************
 *
 * A decorator for the standard {@link PersistenceProvider} which makes sure that the {@link EntityManagerFactory} is 
 * created with the {@link ClassLoaderDecorator}.
 * 
 * @author  Fabrizio Giudici
 * @version $Id: PersistenceProviderDecorator.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public class PersistenceProviderDecorator implements PersistenceProvider
  {
//    private static final String CLASS = PersistenceProviderDecorator.class.getName();
//    private static final Logger logger = Logger.getLogger(CLASS);

    @Nonnull
    private final PersistenceProvider delegate; 
    
    private MultiPersistenceXMLManager multiPersistenceXMLManager;
    
    interface EntityManagerFactoryFactory
      {
        public EntityManagerFactory createEntityManagerFactory();  
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public PersistenceProviderDecorator (final @Nonnull PersistenceProvider delegate)
      {
        this.delegate = delegate;
      }
            
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public EntityManagerFactory createEntityManagerFactory (final @Nonnull String puName,
                                                            final @Nonnull Map properties)
      {
        return createEntityManagerFactory(new EntityManagerFactoryFactory() 
          {
            @Override @Nonnull
            public EntityManagerFactory createEntityManagerFactory() 
              {
                return delegate.createEntityManagerFactory(puName, properties);
              }
          });
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public EntityManagerFactory createContainerEntityManagerFactory (final @Nonnull PersistenceUnitInfo puInfo,
                                                                     final @Nonnull Map properties)
      {
        return createEntityManagerFactory(new EntityManagerFactoryFactory() 
          {
            @Override @Nonnull
            public EntityManagerFactory createEntityManagerFactory() 
              {
                return delegate.createContainerEntityManagerFactory(puInfo, properties);
              }
          });
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    private EntityManagerFactory createEntityManagerFactory (final @Nonnull EntityManagerFactoryFactory factory)
      {
        synchronized (this)
          {
            if (multiPersistenceXMLManager == null)
              {
                multiPersistenceXMLManager = new MultiPersistenceXMLManager();
              }
          }
        
        final Thread currentThread = Thread.currentThread();
        final ClassLoader saveClassLoader = currentThread.getContextClassLoader();
        currentThread.setContextClassLoader(new ClassLoaderDecorator(saveClassLoader, multiPersistenceXMLManager));
        
        try
          {
            return factory.createEntityManagerFactory(); 
          }
        finally
          {
            currentThread.setContextClassLoader(saveClassLoader);
          }
      }
  }
