/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: Persistence.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package javax.persistence;

import javax.annotation.Nonnull;
import java.util.Map;
import javax.persistence.spi.PersistenceProvider;
import org.openide.util.Lookup;
import it.tidalwave.util.logging.Logger;
import it.tidalwave.netbeans.jpa.PersistenceProviderDecorator;

/***********************************************************************************************************************
 *
 * This customization of {@link Persistence} makes sure that a {@link PersistenceProviderDecorator} is used.
 *
 * @author  Fabrizio Giudici
 * @version $Id: Persistence.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public class Persistence 
  {
    private static final String CLASS = Persistence.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);
    
    private static PersistenceProvider persistenceProvider;

    @Nonnull
    public static EntityManagerFactory createEntityManagerFactory (final @Nonnull String persistenceUnitName)
      {
        return createEntityManagerFactory(persistenceUnitName, null);
      }

    @Nonnull
    public static EntityManagerFactory createEntityManagerFactory (final @Nonnull String persistenceUnitName,
                                                                   final @Nonnull Map properties)
      {
        synchronized (Persistence.class)
          {
            if (persistenceProvider == null)
              {
                final PersistenceProvider delegate = Lookup.getDefault().lookup(PersistenceProvider.class);

                if (delegate == null)
                  {
                    throw new PersistenceException("No PersistenceProvider delegate found");
                  }

                logger.info("PersistenceProvider delegate: %s", delegate.getClass());
                persistenceProvider = new PersistenceProviderDecorator(delegate);
              }
          }
        
        final EntityManagerFactory emf = persistenceProvider.createEntityManagerFactory(persistenceUnitName, properties);
        
        if (emf == null)
          {
            throw new PersistenceException("No Persistence provider for EntityManager named " + persistenceUnitName);
          }

        return emf;
      }
  }
