/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: DecoratorNodeWithNoActions.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.nodes;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import javax.swing.Action;
import org.openide.nodes.Node;
import org.openide.util.actions.SystemAction;

/***********************************************************************************************************************
 *
 * A special decorator {@link Node} with no exposed {@link Action}s.
 *
 * @author  Fabrizio Giudici
 * @version $Id: DecoratorNodeWithNoActions.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
@edu.umd.cs.findbugs.annotations.SuppressWarnings("EQ_DOESNT_OVERRIDE_EQUALS")
public class DecoratorNodeWithNoActions extends GenericDecoratorNode
  {
    private final Action[] EMTPY_ACTIONS = new Action[0];
    private final SystemAction[] EMTPY_SYSTEM_ACTIONS = new SystemAction[0];

    public DecoratorNodeWithNoActions (final @Nonnull Node delegate)
      {
        super(delegate);
      }

    @Override @Nonnull
    public final SystemAction[] getActions()
      {
        return EMTPY_SYSTEM_ACTIONS;
      }

    @Override @Nonnull
    public final Action[] getActions (final boolean context)
      {
        return EMTPY_ACTIONS;
      }

    @Override @Nonnull
    public final SystemAction[] getContextActions()
      {
        return EMTPY_SYSTEM_ACTIONS;
      }

    @Override @CheckForNull
    public final SystemAction getDefaultAction()
      {
        return null;
      }
  }
