/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: EnhancedFilterNode.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.nodes;

import javax.annotation.Nonnull;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.awt.Image;
import org.openide.util.Lookup;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;
import it.tidalwave.util.IconProvider;
import it.tidalwave.util.IconProvider.Type;
import static it.tidalwave.util.IconProvider.Type.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: EnhancedFilterNode.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public class EnhancedFilterNode extends FilterNode
  {
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public EnhancedFilterNode (final @Nonnull Node original)
      {
        super(original);
        installIconChangeListener();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public EnhancedFilterNode (final @Nonnull Node original, final @Nonnull Children children)
      {
        super(original, children);
        installIconChangeListener();
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public EnhancedFilterNode (final @Nonnull Node original,
                               final @Nonnull Children children,
                               final @Nonnull Lookup lookup)
      {
        super(original, children, lookup);
        installIconChangeListener();
      }

    // Overrides some behaviours that depend on the Lookup, that could be different than the one of the original Node
    
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public Image getIcon (final int type)
      {
        return getIcon(DEFAULT, type, super.getIcon(type));
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public Image getOpenedIcon (final int type)
      {
        return getIcon(OPEN, type, super.getIcon(type));
      }

    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Nonnull
    private Image getIcon (final @Nonnull Type type, final int size, final @Nonnull Image fallBackImage)
      {
        final Node delegate = getOriginal();
        //
        // If the delegate is not an Enhanced node, it is not using an IconProvider for its icon, thus it is directly
        // providing its own icon. It could even have an IconProvider in its Lookup as it could be a FilterNode; for
        // this reason there's this explicit check.
        //
        if (!(delegate instanceof EnhancedNode) && !(delegate instanceof EnhancedFilterNode))
          {
            return fallBackImage;
          }

        final IconProvider iconProvider = getLookup().lookup(IconProvider.class);
        return (iconProvider != null) ? iconProvider.getIcon(type, 16) : fallBackImage;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private void installIconChangeListener()
      {
        final IconProvider iconProvider = getLookup().lookup(IconProvider.class);
        // FIXME: weak listener? Is the IconProvider allowed to change?

        if (iconProvider != null)
          {
            iconProvider.addPropertyChangeListener(new PropertyChangeListener()
              {
                @Override
                public void propertyChange (final @Nonnull PropertyChangeEvent event)
                  {
                    fireIconChange();
                  }
              });
          }
      }
  }
