/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: GenericDecoratorNode.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.nodes;

import javax.annotation.Nonnull;
import java.util.Arrays;
import org.openide.util.Parameters;
import org.openide.util.Lookup;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;

/***********************************************************************************************************************
 *
 * A specialization of {@link org.openide.nodes.FilterNode} which can be decorated.
 *
 * @author  Fabrizio Giudici
 * @version $Id: GenericDecoratorNode.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public class GenericDecoratorNode extends EnhancedFilterNode
  {
    static class DecoratorChildren extends FilterNode.Children
      {
        @Nonnull
        private final Object[] extraArgs;

        @Nonnull
        private final Class<?>[] argTypes;

        public DecoratorChildren (final @Nonnull Node parent,
                                  final @Nonnull Object ... extraArgs)
          {
            super(parent);
            Parameters.notNull("parent", parent);
            this.extraArgs = extraArgs;

            argTypes = new Class<?>[extraArgs.length + 1];
            argTypes[0] = Node.class;

            // TODO: should be more tolerant and accept any supertype...
            for (int i = 0; i < extraArgs.length; i++)
              {
                argTypes[i + 1] = extraArgs[i].getClass().getInterfaces()[0];
              }
          }

        @Override @Nonnull
        protected Node copyNode (final @Nonnull Node delegate)
          {
            final Class<? extends Node> parentNodeClass = getNode().getClass();
            final Object[] args = new Object[extraArgs.length + 1];
            args[0] = delegate;
            System.arraycopy(extraArgs, 0, args, 1, extraArgs.length);

            try
              {
                return parentNodeClass.getConstructor(argTypes).newInstance(args);
              }
            catch (Exception e)
              {
                // RuntimeError would be caught by the Platform runtime
                throw new Error(parentNodeClass + " must have a public constructor accepting " +
                                Arrays.toString(argTypes), e);
              }
          }
      }

    public GenericDecoratorNode (final @Nonnull Node delegate)
      {
        this(delegate, new Object[0]);
      }

    public GenericDecoratorNode (final @Nonnull Node delegate, 
                                 final @Nonnull Object ... extraArgs)
      {
        super(delegate, new DecoratorChildren(delegate, extraArgs));
      }

    // only useful for subclasses
    protected GenericDecoratorNode (final @Nonnull Node delegate,
                                    final @Nonnull Lookup lookup,
                                    final @Nonnull Object ... extraArgs)
      {
        super(delegate, new DecoratorChildren(delegate, extraArgs), lookup);
      }
  }

