/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: NodeFactorySupport.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.nodes;

import java.util.logging.Logger;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.HashMap;
import java.util.Map;
import java.awt.Image;
import javax.swing.Action;
import org.openide.util.Lookup;
import org.openide.util.ImageUtilities;
import org.openide.util.actions.SystemAction;
import org.openide.util.lookup.Lookups;
import org.openide.nodes.Node;
import it.tidalwave.swing.IconResizer;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: NodeFactorySupport.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 ******************************************************************************/
@Deprecated
public abstract class NodeFactorySupport
  {
    private static final String CLASS = NodeFactorySupport.class.getName();
    private static final Logger logger = Logger.getLogger(CLASS);

    protected final static int ICON_SIZE = 16;

    private final Map<Class, List<Action>> actionsByNodeClass = new HashMap<Class, List<Action>>();

    private final Map<Class, Action> preferredActionByNodeClass = new HashMap<Class, Action>();

    private final Map<Class, Image> imageByNodeClass = new HashMap<Class, Image>();

    private String basePath;

    /***************************************************************************
     *
     *
     **************************************************************************/
    public NodeFactorySupport()
      {
        basePath = getClass().getPackage().getName().replace('.', '/') + "/icons/";
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    public List<Action> findNodeActions (final Class nodeClass)
      {
        List<Action> list = actionsByNodeClass.get(nodeClass);

        if (list == null)
          {
            list = Collections.emptyList();
          }

        logger.fine("findNodeActions(" + nodeClass + ") returning " + list);
        return list;
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    public Action getPreferredNodeAction (final Class nodeClass)
      {
        return preferredActionByNodeClass.get(nodeClass);
      }

    /***************************************************************************
     *
     * @inheritdoc
     *
     **************************************************************************/
    public Image findIcon (final Node node)
      {
        return imageByNodeClass.get(node.getClass());
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    protected void registerIcon (final Class nodeClass, final String iconName)
      {
        imageByNodeClass.put(nodeClass, findIcon(iconName));
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    protected void configureActions (final Class<? extends Node> nodeClass, final String name)
      {
        logger.info("loadActions(" + nodeClass + ", " + name + ")");
        
        final Lookup forPath = Lookups.forPath("NodeActions/" + name);

        for (final Object object : forPath.lookupAll(Object.class))
          {
            if (object instanceof SystemAction)
              {
                registerNodeAction(nodeClass, object.getClass(), false);
                logger.info(">>>> Registered action " + object.getClass() + " to node " + nodeClass);
//                        add(new Button((SystemAction)next));
              }
//            else if (object instanceof JSeparator)
//              {
////                        add((JSeparator)next); TODO
//              }
          }
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    private void registerNodeAction (final Class nodeClass, final Class actionClass, final boolean preferred)
      {
        final Action action = SystemAction.get(actionClass);
        List<Action> list = actionsByNodeClass.get(nodeClass);

        if (list == null)
          {
            list = new ArrayList<Action>();
            actionsByNodeClass.put(nodeClass, list);
          }

        list.add(action);

        if (preferred)
          {
            preferredActionByNodeClass.put(nodeClass, action);
          }
      }

    /***************************************************************************
     *
     * Returns an Image given its name. The icon is searched in the 'icons'
     * subfolder of the folder related to this package.
     *
     * @param  iconName  the name of the resource
     * @return           the image
     *
     **************************************************************************/
    protected Image findIcon (final String iconName)
      {
        return IconResizer.scaledImage(ImageUtilities.loadImage(basePath + iconName), ICON_SIZE);
      }
  }
