/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: DefaultNodeIconProvider.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.nodes.role;

import java.beans.PropertyChangeListener;
import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import java.util.HashMap;
import java.util.Map;
import java.awt.Image;
import org.openide.util.ImageUtilities;
import org.openide.util.Parameters;
import it.tidalwave.swing.IconResizer;
import it.tidalwave.util.IconProvider;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: DefaultNodeIconProvider.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public class DefaultNodeIconProvider implements IconProvider
  {
    @Nonnull
    private final Image baseIcon;

    private final Map<Integer, Image> iconCacheBySize = new HashMap<Integer, Image>();

    public DefaultNodeIconProvider (final @Nonnull String iconPath)
      {
        baseIcon = ImageUtilities.loadImage(iconPath);
        Parameters.notNull("Can't load icon", baseIcon);
      }

    @Override @Nonnull
    public synchronized Image getIcon (final @Nonnull Type type, final @Nonnegative int size)
      {
        Image icon = iconCacheBySize.get(16);

        if (icon == null)
          {
            icon = IconResizer.scaledImage(baseIcon, 16);
            iconCacheBySize.put(16, icon);
          }

        return icon;
      }

    @Override
    public void addPropertyChangeListener (final @Nonnull PropertyChangeListener listener)
      {
        // ignored, the icon is immutable
      }

    @Override
    public void removePropertyChangeListener (final @Nonnull PropertyChangeListener listener)
      {
        // ignored, the icon is immutable
      }
  }
