/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: ErrorNode.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.nodes.role.impl;

import javax.annotation.Nonnull;
import java.awt.Image;
import java.awt.event.ActionEvent;
import javax.swing.Action;
import org.openide.util.HelpCtx;
import org.openide.util.ImageUtilities;
import org.openide.util.RequestProcessor;
import org.openide.util.actions.SystemAction;
import org.openide.util.lookup.Lookups;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: ErrorNode.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public class ErrorNode extends AbstractNode
  {
    @Nonnull
    private final String displayName;

    @Nonnull
    private final Image icon;

    @Nonnull
    private final Retrier retrier;

    private final SystemAction retryAction = new SystemAction()
      {
        @Override
        public void actionPerformed (final @Nonnull ActionEvent event)
          {
            RequestProcessor.getDefault().post(new Runnable()
              {
                @Override
                public void run()
                  {
                    retrier.retry();
                  }
              });
          }

        @Override @Nonnull
        public String getName()
          {
            return "Retry";
          }

        @Override @Nonnull
        public HelpCtx getHelpCtx()
          {
            return HelpCtx.DEFAULT_HELP;
          }
      };
      
    public ErrorNode (final @Nonnull String displayName,
                      final @Nonnull String iconPath,
                      final @Nonnull Throwable t,
                      final @Nonnull Retrier retrier)
      {
        super(Children.LEAF, Lookups.fixed(t));
        this.displayName = displayName;
        this.icon = ImageUtilities.loadImage(iconPath);
        this.retrier = retrier;
      }

    @Override @Nonnull
    public SystemAction[] getActions()
      {
        return new SystemAction[] { retryAction };
      }

    @Override @Nonnull
    public Action getPreferredAction()
      {
        return retryAction;
      }

    @Override @Nonnull
    public String getDisplayName()
      {
        return displayName;
      }

    @Override @Nonnull
    public Image getIcon(int type)
      {
        return icon;
      }

    @Override @Nonnull
    public Image getOpenedIcon(int type)
      {
        return icon;
      }
  }

