/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: NodeDataLoaderTest.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.nodes.role;

import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.lookup.Lookups;
import it.tidalwave.util.NotFoundException;
import org.netbeans.platformx.inject.api.Injector;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;
import static org.hamcrest.CoreMatchers.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: NodeDataLoaderTest.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public class NodeDataLoaderTest
  {
    public static class MyNodeManager implements NodeManager
      {
        private RootNodeProvider rootNodeProvider;

        @Override
        public void setRootNodeProvider (final RootNodeProvider rootNodeProvider)
          {
            this.rootNodeProvider = rootNodeProvider;
          }

        public RootNodeProvider getRootNodeProvider()
          {
            return rootNodeProvider;
          }
      }

    private Node rootNode;
    private MyNodeManager nodeManager;
    private NodeDataLoader fixture;

    @Before
    public void setUp()
      {
        rootNode = new AbstractNode(Children.LEAF);
        nodeManager = spy(new MyNodeManager());
        fixture = new NodeDataLoader() 
          {
            @Override
            protected Node getRootNode() 
              {
                return rootNode;
              }
          };

        Injector.getDefault().inject(fixture, Lookups.fixed(nodeManager));
      }


    @Test
    public void shouldInvokeSetRootNodeProviderAndRetrieveTheCorrectNode()
      throws NotFoundException
      {
        fixture.loadData();
        verify(nodeManager).setRootNodeProvider((RootNodeProvider)any());
        verifyNoMoreInteractions(nodeManager);
        assertThat(nodeManager.rootNodeProvider.getRootNode(), sameInstance(rootNode));
      }
  }