/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package org.netbeans.platformx.inject.api.impl;

import javax.annotation.PostConstruct;
import javax.annotation.Nonnull;
import javax.annotation.Resource;
import javax.inject.Inject;
import javax.inject.Provider;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import org.openide.util.Lookup;
import org.openide.util.lookup.ProxyLookup;
import org.openide.util.lookup.ServiceProvider;
import it.tidalwave.util.logging.Logger;
import java.lang.reflect.Array;
import java.lang.reflect.GenericArrayType;
import javax.annotation.CheckForNull;
import org.netbeans.platformx.inject.api.Injector;
import org.netbeans.platformx.inject.api.annotation.Mandatory;

/**
 *
 * @author fritz
 */
@ServiceProvider(service=Injector.class)
public class DefaultInjector extends Injector
  {
    private final static String CLASS = DefaultInjector.class.getName();
    private final static Logger logger = Logger.getLogger(CLASS);

    @Override
    public void inject (final @Nonnull Object object, final @Nonnull Lookup lookup)
      {
        injectLookup(object, lookup);
        callPostConstructors(object);
      }

    @Override
    public void inject (final @Nonnull Collection<Object> objects2,
                        final @Nonnull Lookup lookup)
      {
        final List<Object> objects = new ArrayList<Object>(objects2);
        final Map<Object, Set<Class<?>>> dependencyMap = new HashMap<Object, Set<Class<?>>>();

        for (final Object role : objects)
          {
            dependencyMap.put(role, findDependencies(role));
          }

        logger.finest(">>>> dependency map: %s", dependencyMap);

        // FIXME: this doesn't detect cyclical dependencies - do they cause troubles to sort?

        Collections.sort(objects, new Comparator<Object>()
          {
            @Override
            public int compare (final @Nonnull Object o1, final @Nonnull Object o2)
              {
                final Set<Class<?>> dependencies = dependencyMap.get(o1);
                return containsSuperClass(dependencies, o2.getClass()) ? -1 : +1;
              }
          });

        for (final Object role : objects)
          {
            injectLookup(role, lookup);
          }

        for (final Object role : objects)
          {
            logger.finer(">>>> initializing: %s", role);
            callPostConstructors(role);
          }
      }

    /*******************************************************************************************************************
     *
     * Returns true if a collection of classes contains the given class or a superclass of it, or a superinterface of
     * it.
     *
     * @param  classes  the collection that is searched
     * @param  clazz    the class to search
     * @return          true if the class is found
     *
     ******************************************************************************************************************/
    private static boolean containsSuperClass (final Collection<Class<?>> classes, final Class<?> clazz)
      {
        if (classes.contains(clazz))
          {
            return true;
          }

        final Class<?> superclass = clazz.getSuperclass();

        if ((superclass != null) && containsSuperClass(classes, superclass))
          {
            return true;
          }

        for (Class<?> interf : clazz.getInterfaces())
          {
            if (containsSuperClass(classes, interf))
              {
                return true;
              }
          }

        return false;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void injectLookup (final @Nonnull Object role,
                              final @Nonnull Lookup injectedLookup)
      {
        injectLookup(role, role.getClass(), new ProxyLookup(injectedLookup, Lookup.getDefault()));
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public Set<Class<?>> findDependencies (final @Nonnull Object role)
      {
        return findDependencies(role, role.getClass());
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @edu.umd.cs.findbugs.annotations.SuppressWarnings("REC_CATCH_EXCEPTION")
    private void injectLookup (final @Nonnull Object role,
                               final @Nonnull Class<?> clazz,
                               final @Nonnull Lookup injectedLookup)
      {
        logger.finest("injectLookup(%s, %s, ...)", role, clazz);
        assert injectedLookup != null : "injectedLookup is null";

        final Class<?> superclass = clazz.getSuperclass();

        if (superclass != null)
          {
            injectLookup(role, superclass, injectedLookup);
          }

        for (final Field field : clazz.getDeclaredFields())
          {
            if ((field.getAnnotation(Resource.class) != null) || (field.getAnnotation(Inject.class) != null))
              {
                logger.finest(">>>> injecting field %s...", field);

                Class<?> fieldType = field.getType();

                try
                  {
                    Object toInject = null;

                    if (fieldType.equals(Lookup.class))
                      {
                        toInject = injectedLookup;
                      }
                    else if (fieldType.equals(Provider.class))
                      {
//                        final ParameterizedType genericType = (ParameterizedType)field.getGenericType();
//                        final Type x = genericType.getActualTypeArguments()[0];
//                        final Class<?> type = getClass(field.getGenericType());
//                        toInject = new LookupProvider(injectedLookup, type);
                        toInject = createProvider(field, injectedLookup);
                        logger.finest(">>>>>>>> Injecting provider for %s", toInject);
                      }
                    else
                      {
                        toInject = createProvider(field, injectedLookup).get();
                      }

                    if (toInject == null)
                      {
                        logger.fine("Can't lookup resource: %s", fieldType);
                      }

                    logger.finest(">>>>>>>> injected object is: %s", toInject);

                    if ((toInject == null) && (field.getAnnotation(Mandatory.class) != null))
                      {
                        throw new RuntimeException("Mandatory dependency not found for " + field);
                      }

                    field.setAccessible(true);
                    field.set(role, toInject);
                  }
                catch (RuntimeException e)
                  {
                    throw e;
                  }
                catch (Exception e)
                  {
                    logger.severe("While injecting %s to %s: %s", fieldType, role, e);
                    logger.throwing(CLASS, "injectLookup()", e);
                    throw new RuntimeException(e);
                  }
              }
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private Provider<?> createProvider (final @Nonnull Field field,
                                        final @Nonnull Lookup lookup)
      {
        final Class<?> type = getClass(field.getGenericType());
        
        if (!Collection.class.isAssignableFrom(field.getType()))
          {
            return new LookupProvider(lookup, type);
          }
        else
          {
//            final ParameterizedType genericType = (ParameterizedType)field.getGenericType();
//            final Type itemClass = genericType.getActualTypeArguments()[0];
            return new LookupAllProvider(lookup, type);
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private static Set<Class<?>> findDependencies (final @Nonnull Object role,
                                                   final @Nonnull Class<?> clazz)
      {
        final Set<Class<?>> result = new TreeSet<Class<?>>(new Comparator<Class<?>>()
          {
            @Override
            public int compare (final @Nonnull Class<?> c1, final @Nonnull Class<?> c2)
              {
                return c1.getName().compareTo(c2.getName());
              }
          });

        final Class<?> superclass = clazz.getSuperclass();

        if (superclass != null)
          {
            result.addAll(findDependencies(role, superclass));
          }

        for (final Field field : clazz.getDeclaredFields())
          {
            if (field.getAnnotation(Resource.class) != null)
              {
                final Class<?> fieldType = field.getType();
                result.add(fieldType);
              }
          }

        return result;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private void callPostConstructors (final @Nonnull Object object)
      {
        callPostConstructors(object, object.getClass());
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private void callPostConstructors (final @Nonnull Object object,
                                       final @Nonnull Class<?> clazz)
      {
        final Class<?> superclass = clazz.getSuperclass();

        if (superclass != null)
          {
            callPostConstructors(object, superclass);
          }

        for (final Method method : clazz.getDeclaredMethods())
          {
            if (method.getAnnotation(PostConstruct.class) != null)
              {
                try
                  {
                    method.setAccessible(true);
                    method.invoke(object);
                  }
                catch (Exception e)
                  {
                    logger.severe("While initializing %s: %s", object, e);
                    logger.throwing(CLASS, "callPostConstructors()", e);
                    e.printStackTrace();
                  }
              }
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @CheckForNull // FIXME: adapted from ReflectionUtils
    public static Class<?> getClass (final @Nonnull Type type)
      {
        if (type instanceof Class<?>)
          {
            return (Class<?>)type;
          }
        else if (type instanceof ParameterizedType)
          {
            final ParameterizedType genericType = (ParameterizedType)type;
            return getClass(genericType.getActualTypeArguments()[0]);
//            return getClass(((ParameterizedType)type).getRawType());
          }
        else if (type instanceof GenericArrayType)
          {
            final Type componentType = ((GenericArrayType)type).getGenericComponentType();
            final Class<?> componentClass = getClass(componentType);

            if (componentClass == null)
              {
                return null;
              }

            return Array.newInstance(componentClass, 0).getClass();
          }
        else
          {
            return null;
          }
      }
  }
