/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: DefaultInjectorTest.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package org.netbeans.platformx.inject.api.impl;

import javax.inject.Inject;
import javax.inject.Provider;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import org.openide.util.Lookup;
import org.openide.util.lookup.Lookups;
import org.netbeans.platformx.inject.api.Injector;
import it.tidalwave.netbeans.util.test.TestLoggerSetup;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.netbeans.platformx.inject.api.annotation.Mandatory;
import static org.junit.Assert.*;
import static org.hamcrest.CoreMatchers.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: DefaultInjectorTest.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public class DefaultInjectorTest
  {
    private Injector fixture;

    private final String string = "string";
    private final Date date = new Date();
    private final DateFormat dateFormat = new SimpleDateFormat();

    private Lookup lookup;

    @BeforeClass
    public static void setupLogging()
      {
        TestLoggerSetup.setupLogging(DefaultInjectorTest.class);
      }

    @Before
    public void setupFixture()
      {
        fixture = new DefaultInjector();
      }

    public static class Role
      {
        @Inject
        private Date dateField;

        @Inject
        private String stringField;

        @Inject
        private Provider<Date> dateProviderField;

        @Inject
        private List<Integer> listField;

        @Inject
        private Provider<List<Integer>> listProviderField;

        @Inject
        private Lookup lookupField;
        
        public Date getDateField()
          {
            return dateField;
          }

        public String getStringField()
          {
            return stringField;
          }
      }

    public static class RoleSubClass extends Role
      {
        @Inject
        private DateFormat dateFormatField;

        public DateFormat getDateFormat()
          {
            return dateFormatField;
          }
      }

    public static class Role2
      {
        @Inject
        private String string;
      }

    public static class Role3
      {
        @Inject @Mandatory
        private Date date;
      }

    @Before
    public void setupLookup()
      {
        lookup = Lookups.fixed(string, date, dateFormat,  1, 2, 3, 4);
      }

    @Test
    public void mustInjectFieldsInASimpleClass()
      throws Exception
      {
        final Role role = new Role();
        fixture.inject(role, lookup);

        assertThat(role.dateField, is(sameInstance(date)));
        assertThat(role.stringField, is(sameInstance(string)));
        assertThat(role.dateProviderField, is(instanceOf(LookupProvider.class)));
        assertThat(((LookupProvider<Date>)role.dateProviderField).getProvidedClass(), is(sameInstance(Date.class)));
        assertThat(role.dateProviderField.get(), is(sameInstance(date)));
        assertThat(role.listField, is(Arrays.asList(1, 2, 3, 4)));
//        assertThat(role.field5, is(instanceOf(LookupProvider.class)));
//        assertThat(role.field5.get(), is(Arrays.asList(1, 2, 3, 4)));

        assertThat(role.lookupField, is(notNullValue()));
        // TODO: assert that lookup includes our lookup and the default lookup
      }

    @Test
    public void mustInjectFieldsInASuperClass()
      throws Exception
      {
        final RoleSubClass role = new RoleSubClass();
        fixture.inject(role, lookup);

        // TODO assert all fields
        assertThat(role.getDateField(), is(sameInstance(date)));
        assertThat(role.getStringField(), is(sameInstance(string)));
        assertThat(role.dateFormatField, is(sameInstance(dateFormat)));
      }

    @Test
    public void mustSetNullReferenceWhenNoDependencyIsFound()
      {
        final Role2 role = new Role2();
        fixture.inject(role, Lookup.EMPTY);
        assertThat(role.string, is(nullValue()));
      }

    @Test(expected=RuntimeException.class)
    public void mustThrowExceptionWhenAMandatoryDependencyIsNotFound()
      {
        final Role3 role = new Role3();
        fixture.inject(role, Lookup.EMPTY);
      }

    @Test
    public void test_OPENBLUESKY_1()
      throws Exception
      {
        final Lookup lookup = Lookups.fixed(string);

        final Role role = new Role();
        fixture.inject(role, lookup);

        assertThat(role.dateField, is(nullValue()));
        assertThat(role.stringField, is(sameInstance(string)));
      }
 }