/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: IdFactoryDelegate.java,v b34faa93e377 2010/04/03 13:17:08 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.util;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import it.tidalwave.util.Id;
import it.tidalwave.netbeans.util.impl.ReflectionUtils;

/***********************************************************************************************************************
 *
 * This class is responsible for creating {@link Id}s for a given class or family of classes. It is invoked by the
 * {@link DefaultIdFactory}.
 * 
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public abstract class IdFactoryDelegate<T>
  {
    @Nonnull
    private final Class<T> managedClass = (Class<T>)ReflectionUtils.getTypeArguments(IdFactoryDelegate.class, getClass()).get(0);

    /*******************************************************************************************************************
     *
     * Returns the class managed by this delegate.
     *
     * @return  the managed class
     *
     ******************************************************************************************************************/
    @Nonnull
    public final Class<T> getManagedClass()
      {
        return managedClass;
      }

    /*******************************************************************************************************************
     *
     * This method must create a new {@link Id} for the managed class. It is possible that a reference to the newly
     * created object needing the {@link Id} is passed and it might eventually be used as a parameter for the
     * computation of the {@link Id}.
     *
     * @param  object  the object needing the {@code Id}
     * @return         the new {@code Id}
     *
     ******************************************************************************************************************/
    @Nonnull
    public abstract Id createId (final @CheckForNull Object object);
  }
