/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: LocatorTest.java,v c31d06a10003 2010/04/02 22:25:13 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.util;

import javax.annotation.Nonnull;
import java.lang.reflect.InvocationTargetException;
import java.util.concurrent.atomic.AtomicReference;
import java.awt.EventQueue;
import javax.inject.Provider;
import org.junit.BeforeClass;
import org.junit.Test;
import org.openide.util.Lookup;
import static org.junit.Assert.*;
import static org.hamcrest.CoreMatchers.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: LocatorTest.java,v c31d06a10003 2010/04/02 22:25:13 fabrizio $
 *
 **********************************************************************************************************************/
public class LocatorTest 
  {
    @BeforeClass
    public static void installMockServices()
      {
//        MockLookup.setInstances();
      }

    @Test
    public void mustFindAService()
      {
        final MockService service = Locator.find(MockService.class);
        assertThat(service, is(notNullValue()));
        assertThat(service, is(sameInstance(Lookup.getDefault().lookup(MockService.class))));
      }

    @Test(expected=Locator.NotFoundException.class)
    public void mustThrowNotFoundExceptionWhenSearchesAnUnknownService()
      {
        Locator.find(UnregisteredMockService.class);
      }

    @Test
    public void mustFindAServiceByProvider()
      {
        final Provider<MockService> serviceProvider = Locator.createProviderFor(MockService.class);
        assertThat(serviceProvider, is(notNullValue()));
        assertThat(serviceProvider.get(), is(notNullValue()));
        assertThat(serviceProvider.get(), is(sameInstance(Lookup.getDefault().lookup(MockService.class))));
      }

    @Test
    public void mustReturnAValidProviderEvenForAnUnregisteredService()
      {
        final Provider<UnregisteredMockService> serviceProvider = Locator.createProviderFor(UnregisteredMockService.class);
        assertThat(serviceProvider, is(notNullValue()));
      }

    @Test(expected=Locator.NotFoundException.class)
    public void mustThrowNotFoundExceptionWhenUsingAProviderForAnUnregisteredService()
      {
        final Provider<UnregisteredMockService> serviceProvider = Locator.createProviderFor(UnregisteredMockService.class);
        assertThat(serviceProvider, is(notNullValue()));
        serviceProvider.get();
      }

    @Test
    public void mustFindATopComponentFromOutsideTheEDT()
      throws InterruptedException, InvocationTargetException
      {
        final MockTopComponent topComponent = Locator.find(MockTopComponent.class);
        assertThat(topComponent, is(sameInstance(findTopComponent())));
      }

    @Test
    public void mustFindATopComponentFromTheEDT()
      throws InterruptedException, InvocationTargetException
      {
        final AtomicReference<MockTopComponent> topComponent = new AtomicReference<MockTopComponent>(null);

        EventQueue.invokeAndWait(new Runnable()
          {
            @Override
            public void run()
              {
                topComponent.set(Locator.find(MockTopComponent.class));
              }
          });

        assertNotNull(topComponent.get());
        assertThat(topComponent.get(), is(sameInstance(findTopComponent())));
      }

    @Test
    public void mustFindATopComponentByInterfaceFromOutsideTheEDT()
      throws InterruptedException, InvocationTargetException
      {
        final MockTopComponentInterface topComponent = Locator.find(MockTopComponentInterface.class);
        assertThat(topComponent, is(sameInstance((MockTopComponentInterface)findTopComponent())));
      }

    @Test
    public void mustFindATopComponentByInterfaceFromTheEDT()
      throws InterruptedException, InvocationTargetException
      {
        final AtomicReference<MockTopComponentInterface> topComponent = new AtomicReference<MockTopComponentInterface>(null);

        EventQueue.invokeAndWait(new Runnable()
          {
            @Override
            public void run()
              {
                topComponent.set(Locator.find(MockTopComponentInterface.class));
              }
          });

        assertNotNull(topComponent.get());
        assertThat(topComponent.get(), is(sameInstance((MockTopComponentInterface)findTopComponent())));
      }

    @Nonnull
    private MockTopComponent findTopComponent()
      throws InterruptedException, InvocationTargetException
      {
        final AtomicReference<MockTopComponent> topComponent = new AtomicReference<MockTopComponent>(null);

        EventQueue.invokeAndWait(new Runnable()
          {
            @Override
            public void run()
              {
                topComponent.set(MockTopComponent.findInstance());
              }
          });

        assertNotNull(topComponent.get());
        return topComponent.get();
      }
  }
