/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: LookupDelegatingToProviderTest.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.util;

import javax.inject.Provider;
import java.util.Date;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;
import static org.hamcrest.CoreMatchers.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: LookupDelegatingToProviderTest.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public class LookupDelegatingToProviderTest
  {
    public static class MockClass
      {
      }

    private LookupDelegatingToProvider<MockClass> fixture;
    private Provider<MockClass> provider;
    private MockClass product;

    @Before
    public void setUp()
      {
        product = new MockClass();
        provider = mock(Provider.class);
        when(provider.get()).thenReturn(product);
        fixture = new LookupDelegatingToProvider<MockClass>(MockClass.class, provider);
      }

    @Test
    public void shouldCallTheProviderWhenLookingUpForTheProperClass()
      {
        final MockClass result = fixture.lookup(MockClass.class);
        verify(provider, times(1)).get();
        verifyNoMoreInteractions(provider);
        assertThat(result, is(sameInstance(product)));
      }

    @Test
    public void shouldJustReturnNullWhenLookingUpAllTheOtherClasses()
      {
        assertThat(fixture.lookup(String.class), is(nullValue()));
        assertThat(fixture.lookup(Date.class), is(nullValue()));
        assertThat(fixture.lookup(Integer.class), is(nullValue()));
      }
  }