/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: MockTopComponent.java,v 68b42c2f9eb0 2010/04/02 21:32:14 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.util;

import javax.annotation.Nonnull;
import java.awt.EventQueue;
import org.openide.windows.TopComponent;
import org.openide.windows.WindowManager;
import static org.junit.Assert.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: MockTopComponent.java,v 68b42c2f9eb0 2010/04/02 21:32:14 fabrizio $
 *
 **********************************************************************************************************************/
public class MockTopComponent extends TopComponent implements MockTopComponentInterface 
  {
    private static final String PREFERRED_ID = "MockTopComponent";

    private static MockTopComponent instance;

    protected MockTopComponent()
      {
        assertTrue(EventQueue.isDispatchThread());
      }

    @Override @Nonnull
    public String preferredID()
      {
        return PREFERRED_ID;
      }

    @Nonnull
    public static synchronized MockTopComponent getDefault()
      {
        assertTrue(EventQueue.isDispatchThread());

        if (instance == null)
          {
            instance = new MockTopComponent();
          }

        return instance;
      }

    @Nonnull
    public static synchronized MockTopComponent findInstance()
      {
        assertTrue(EventQueue.isDispatchThread());

        final TopComponent component = WindowManager.getDefault().findTopComponent(PREFERRED_ID);

        if (component == null)
          {
            System.err.println("Cannot find component. It will not be located properly in the window system. " +
                               PREFERRED_ID);
            return getDefault();
          }

        if (component instanceof MockTopComponent)
          {
            return (MockTopComponent)component;
          }

        System.err.println("There seem to be multiple components with the '%s' ID. That is a potential source of errors and " +
                           "unexpected behavior." + PREFERRED_ID);
        return getDefault();
      }
  }
