/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: DropAcceptor.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.visual;

import javax.annotation.Nonnull;
import java.util.Collection;
import java.awt.dnd.DropTargetDropEvent;
import org.openide.nodes.Node;

/***********************************************************************************************************************
 *
 * Objects implementing this interface should be registeted to the {@link EnhancedSceneView} in order to accept
 * dropped objects of certain types. They are registered by means of the
 * {@link #registerDropAcceptor(java.awt.datatransfer.DataFlavor,
 * it.tidalwave.netbeans.scene.NodeScene.DropAcceptor)} and associated to a certain {@link DataFlavor}.
 *
 * @author  Fabrizio Giudici
 * @version $Id: DropAcceptor.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public interface DropAcceptor
  {
    /***************************************************************************************************************
     *
     * Processes a drop event and creates one or more {@link Node}s that  should be added to the {@link NodeScene}.
     * If the object is not accepted, an empty collection must be returned.
     *
     * @param  event   the drop event
     * @param  object  the drop object, extracted from the {@link Transferable}
     *
     **************************************************************************************************************/
    @Nonnull
    public Collection<Node> process (@Nonnull DropTargetDropEvent event, @Nonnull Object object);
  }
