/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: EnhancedWindowManager.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.windows;

import javax.annotation.Nonnull;
import javax.annotation.concurrent.ThreadSafe;
import javax.swing.JFrame;

/***********************************************************************************************************************
 *
 * This class provides some extra capabilities for manipulating the main window.
 * 
 * @author  Fabrizio Giudici
 * @version $Id: EnhancedWindowManager.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
@ThreadSafe
public interface EnhancedWindowManager
  {
    /*******************************************************************************************************************
     *
     * Sets the prefix of the main window title. This is usually changed only once with the name of the application.
     * 
     * @param   titlePrefix  the title prefix
     *
     ******************************************************************************************************************/
    public void setTitlePrefix (@Nonnull String titlePrefix);
    
    /*******************************************************************************************************************
     *
     * Returns the title prefix.
     * 
     * @return  the title prefix
     *
     ******************************************************************************************************************/
    @Nonnull
    public String getTitlePrefix();
    
    /*******************************************************************************************************************
     *
     * Sets the title of the main window. It will appear after the title prefix
     * (which is not changed by this method).
     * 
     * @param  title  the title
     *
     ******************************************************************************************************************/
    public void setTitle (@Nonnull String title);
    
    /*******************************************************************************************************************
     *
     * Returns the title.
     * 
     * @return  the title
     *
     ******************************************************************************************************************/
    @Nonnull
    public String getTitle();
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public JFrame getMainWindow();
    
    /*******************************************************************************************************************
     *
     * Creates a floating panel.
     * 
     * @param  contents   the contents of the pane
     * @param  title      the title of the pane
     *
     ******************************************************************************************************************/
//    public FloatingPane createFloatingPane (final JPanel contents, final String title);
  }
