/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: EnhancedWindowManagerImpl.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.windows.impl;

import javax.annotation.Nonnull;
import javax.annotation.concurrent.ThreadSafe;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import org.openide.windows.WindowManager;
import it.tidalwave.netbeans.windows.EnhancedWindowManager;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: EnhancedWindowManagerImpl.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
@ThreadSafe
public class EnhancedWindowManagerImpl implements EnhancedWindowManager
  {
    @Nonnull
    private String titlePrefix = "";

    @Nonnull
    private String title = "";
    
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    public void setTitlePrefix (final @Nonnull String titlePrefix)
      {
        this.titlePrefix = titlePrefix;  
        internalSetTitle();
      }
    
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public String getTitlePrefix()
      {
        return titlePrefix;  
      }
    
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public void setTitle (final @Nonnull String title)
      {
        this.title = title;  
        internalSetTitle();
      }
    
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public String getTitle()
      {
        return title;  
      }
    
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public JFrame getMainWindow()
      {
        return (JFrame)WindowManager.getDefault().getMainWindow();
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
//    public FloatingPane createFloatingPane (final JPanel contents, final String title)
//      {
//        final JFrame jFrame = (JFrame)WindowManager.getDefault().getMainWindow();
//        final FloatingPane floatingPane = new FloatingPaneImpl(jFrame, contents);
//        floatingPane.setTitle(title);
//        floatingPane.setVisible(true);
//        
//        return floatingPane;
//      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private void internalSetTitle()
      {
        if (SwingUtilities.isEventDispatchThread())
          {
            internalSetTitle2();  
          }
        
        else
          {
            SwingUtilities.invokeLater(new Runnable() 
              {
                @Override
                public void run() 
                  {
                    internalSetTitle2();  
                  }
              });   
          }
      }
    
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    private void internalSetTitle2()
      {
        WindowManager.getDefault().getMainWindow().setTitle(fixed(titlePrefix) + 
                (((titlePrefix != null) && (title != null)) ? " - " : "") + fixed(title));
      }
    
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Nonnull 
    private String fixed (final @Nonnull String string)
      {
        return (string != null) ? string : "";  
      }
  }
