/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: ToggleComponentAction.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.windows.impl.action;

import javax.annotation.Nonnull;
import java.awt.EventQueue;
import java.awt.event.ActionEvent;
import javax.swing.AbstractAction;
import org.openide.filesystems.FileObject;
import org.openide.windows.TopComponent;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: ToggleComponentAction.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public class ToggleComponentAction extends AbstractAction
        // extends BooleanStateAction
        // FIXME: triggers java.lang.IllegalStateException: Warning: multiple instances of shared class it.tidalwave.netbeans.windows.impl.action.ToggleComponentAction created.
  {
    private TopComponent component;
    private final String displayName;

    public ToggleComponentAction (@Nonnull final FileObject fileObject)
      {
        displayName = (String)fileObject.getAttribute("displayName");
        putValue(NAME, displayName);
        
        EventQueue.invokeLater(new Runnable()
          {
            @Override
            public void run()
              {
                component = (TopComponent)fileObject.getAttribute("component");
              }
          });
      }

    @Override
    final public void actionPerformed (final @Nonnull ActionEvent event)
      {
        EventQueue.invokeLater(new Runnable()
          {
            @Override
            public void run()
              {
                if (component != null)
                  {
                    if (!component.isOpened())
                      {
                        component.open();
                        component.requestActive();
                      }

                    else if (!component.isVisible())
                      {
                        component.requestActive();
                      }

                    else
                      {
                        component.close();
                      }
                  }
              }
          });
      }
//
//    @Override
//    @Nonnull
//    public String getName()
//      {
//        return displayName;
//      }
//
//    @Override
//    @Nonnull
//    public HelpCtx getHelpCtx()
//      {
//        return HelpCtx.DEFAULT_HELP;
//      }
  }