/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: TitleActivator.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.windows.role;

import javax.annotation.Nonnull;
import javax.inject.Inject;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import org.openide.windows.TopComponent;
import it.tidalwave.netbeans.windows.EnhancedWindowManager;

/***********************************************************************************************************************
 *
 * This role sets the title of the main window in function of the name of the associated {@link TopComponent}.
 *
 * @stereotype Role
 *
 * @author  Fabrizio Giudici
 * @version $Id: TitleActivator.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public class TitleActivator extends TopComponentRoleSupport
  {
    protected String pattern = "%s";

    @Inject
    private EnhancedWindowManager windowManager;

    @Inject
    private TopComponent topComponent;

    private final PropertyChangeListener displayNameListener = new PropertyChangeListener()
      {
        @Override
        public void propertyChange (final @Nonnull PropertyChangeEvent event)
          {
            windowManager.setTitle(String.format(pattern, topComponent.getName()));
          }
      };

    public void setPattern (final @Nonnull String pattern)
      {
        this.pattern = pattern;
      }

    @Nonnull
    public String getPattern()
      {
        return pattern;
      }

    @Override
    public void notifyActivated()
      {
        windowManager.setTitle(String.format(pattern, topComponent.getName()));
        topComponent.addPropertyChangeListener(displayNameListener);
      }

    @Override
    public void notifyDeactivated()
      {
        topComponent.removePropertyChangeListener(displayNameListener);
        windowManager.setTitle(null);
      }
  }
