/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: TitleActivatorTest.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.windows.role;

import java.beans.PropertyVetoException;
import org.openide.util.lookup.Lookups;
import org.openide.windows.TopComponent;
import it.tidalwave.netbeans.windows.EnhancedWindowManager;
import org.netbeans.platformx.inject.api.Injector;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;
import static org.mockito.Mockito.*;
import static org.hamcrest.CoreMatchers.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: TitleActivatorTest.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
public class TitleActivatorTest
  {
    private TopComponent topComponent;
    private EnhancedWindowManager windowManager;
    private TitleActivator fixture;

    @Before
    public void setup()
      {
        topComponent = new TopComponent();
        topComponent.setName("name1");
        windowManager = mock(EnhancedWindowManager.class);
        fixture = new TitleActivator();
        Injector.getDefault().inject(fixture, Lookups.fixed(topComponent, windowManager));
        verifyZeroInteractions(windowManager);
      }

    @Test
    public void shouldHaveIdentityPatternByDefault()
      {
        assertThat(fixture.getPattern(), is("%s"));
      }

    @Test
    public void shouldDoNothingAfterCreation()
      throws PropertyVetoException
      {
        topComponent.setName("name2");
        verifyNoMoreInteractions(windowManager);
      }

    @Test
    public void shouldChangeTitleWhenActive()
      throws PropertyVetoException
      {
        fixture.setPattern("PRE %s POST");
        
        fixture.notifyActivated();
        verify(windowManager).setTitle(eq("PRE name1 POST"));
        
        topComponent.setName("name2");
        // called twice because of https://netbeans.org/bugzilla/show_bug.cgi?id=177634
        verify(windowManager, times(2)).setTitle(eq("PRE name2 POST")); 

        topComponent.setName("name3");
        verify(windowManager, times(2)).setTitle(eq("PRE name2 POST"));
      }

    @Test
    public void shouldSetNullTitleWhenInactivatedAndThenDoNothingMore()
      throws PropertyVetoException
      {
        fixture.setPattern("PRE %s POST");

        fixture.notifyActivated();
        verify(windowManager).setTitle(eq("PRE name1 POST"));
        
        fixture.notifyDeactivated();
        verify(windowManager).setTitle(argThat(nullValue(String.class)));

        topComponent.setName("name2");
        verifyZeroInteractions(windowManager);

        topComponent.setName("name3");
        verifyZeroInteractions(windowManager);
      }

  }