/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: AutomatedTestNode.java,v 8f602be640c5 2010/03/28 22:35:05 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.automatedtest;

import java.awt.EventQueue;
import java.awt.Image;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashMap;
import java.util.Map;
import javax.annotation.Nonnull;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.ImageUtilities;
import org.openide.util.lookup.Lookups;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: AutomatedTestNode.java,v 8f602be640c5 2010/03/28 22:35:05 fabrizio $
 *
 **********************************************************************************************************************/
public class AutomatedTestNode extends AbstractNode
  {
    private static final Map<AutomatedTest.Status, Image> ICON_MAP = new HashMap<AutomatedTest.Status, Image>();

    static
      {
        register(AutomatedTest.Status.NOT_RUN, "kservices.png");
        register(AutomatedTest.Status.RUNNING, "ajax-loader.gif");
        register(AutomatedTest.Status.PASSED, "apply.png");
        register(AutomatedTest.Status.FAILED, "error.png");
      }

    private static void register (@Nonnull final AutomatedTest.Status status, @Nonnull final String name)
      {
        final String path = AutomatedTestNode.class.getPackage().getName().replace('.', '/') + "/icons/";
        ICON_MAP.put(status, ImageUtilities.loadImage(path + name));
      }

    @Nonnull
    private final AutomatedTest test;

    public AutomatedTestNode (@Nonnull final AutomatedTest test)
      {
        this(Children.LEAF, test);
      }

    public AutomatedTestNode (@Nonnull final Children children,
                              @Nonnull final AutomatedTest test)
      {
        super(children, Lookups.singleton(test));
        this.test = test;

        test.addPropertyChangeListener(new PropertyChangeListener()
          {
            public void propertyChange (@Nonnull final PropertyChangeEvent event)
              {
                EventQueue.invokeLater(new Runnable()
                  {
                    public void run()
                      {
                        propagateIconChange();
                      }
                  });
              }
          });
      }

    protected void propagateIconChange()
      {
        fireIconChange();
        final Node parentNode = getParentNode();

        if (parentNode instanceof AutomatedTestNode)
          {
            ((AutomatedTestNode)parentNode).propagateIconChange();
          }
      }

    @Override
    @Nonnull
    public Image getIcon (final int type)
      {
        return ICON_MAP.get(test.getStatus());
      }

    @Override
    @Nonnull
    public Image getOpenedIcon (final int type)
      {
        return getIcon(type);
      }

    @Override
    @Nonnull
    public String getName()
      {
        return test.getName();
      }

    @Override
    @Nonnull
    public String getDisplayName()
      {
        return test.getName();
      }
  }
