/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: AutomatedTestSuite.java,v 8f602be640c5 2010/03/28 22:35:05 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.automatedtest;

import javax.annotation.Nonnull;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Comparator;
import java.util.Iterator;
import java.util.SortedSet;
import java.util.TreeSet;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: AutomatedTestSuite.java,v 8f602be640c5 2010/03/28 22:35:05 fabrizio $
 *
 **********************************************************************************************************************/
public class AutomatedTestSuite extends AutomatedTest implements Iterable<AutomatedTest>
  {
    private final String name;

    private final Comparator<AutomatedTest> comparator = new Comparator<AutomatedTest>()
      {
        public int compare (@Nonnull final AutomatedTest t1,
                            @Nonnull final AutomatedTest t2)
          {
            return t1.getName().compareTo(t2.getName());
          }
      };

    private final SortedSet<AutomatedTest> tests = new TreeSet<AutomatedTest>(comparator);

    private final PropertyChangeListener propertyChangeListener = new PropertyChangeListener()
      {
        public void propertyChange (final PropertyChangeEvent event)
          {
            AutomatedTestSuite.this.propertyChangeSupport.firePropertyChange(PROP_STATUS, null, getStatus());
          }
      };
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    public AutomatedTestSuite (final String name)
      {
        this.name = name;
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    @Override
    public String getName()
      {
        return name;  
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    @Override
    public Status getStatus()
      {
        Status result = Status.PASSED;

        for (final AutomatedTest test : tests)
          {
            if (test.getStatus() == Status.RUNNING)
              {
                result = Status.RUNNING;
                break;
              }
            
            if (test.getStatus() == Status.FAILED)
              {
                result = Status.FAILED;
                break;
              }

            if (test.getStatus() == Status.NOT_RUN)
              {
                result = Status.NOT_RUN;
              }
          }

        return result;
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    public void add (final AutomatedTest test)
      {
        test.addPropertyChangeListener(propertyChangeListener);
        tests.add(test);  
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    public Iterator<AutomatedTest> iterator() 
      {
        return tests.iterator();
      }
  }
