/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: ComponentFinder.java,v 8f602be640c5 2010/03/28 22:35:05 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.automatedtest;

import java.lang.reflect.Method;
import java.util.logging.Logger;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dialog;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JRootPane;
import javax.swing.SwingUtilities;
import org.openide.windows.TopComponent;
import static org.junit.Assert.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: ComponentFinder.java,v 8f602be640c5 2010/03/28 22:35:05 fabrizio $
 *
 **********************************************************************************************************************/
public final class ComponentFinder
  {
    private static final String CLASS = ComponentFinder.class.getName();    
    private static final Logger logger = Logger.getLogger(CLASS);
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    private ComponentFinder()
      {
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    public static <T extends TopComponent> T findTopComponent (final Class<T> tcClass)
      {
        final TopComponent[] result = new TopComponent[1];
        
        try 
          {
            SwingUtilities.invokeAndWait(new Runnable() 
              {
                public void run() 
                  {
                    try 
                      {
                        final Method method = tcClass.getMethod("findInstance");
                        result[0] = (TopComponent)method.invoke(null);
                      } 
                    catch (Exception e) 
                      {
                      }
                  }
              });
          } 
        catch (Exception e) 
          {
            logger.throwing(CLASS, "", e);
          }
        
        final T t = (T)result[0];
        
        if (t == null)
          {
            throw new AssertionError("Cannot find TopComponent for class: " + tcClass);
          }
        
        return t;
      }
    
   /***************************************************************************
     *
     * Finds a button given the bound action name.
     *
     **************************************************************************/
    public static <T extends Component> T findComponentByName (final Container container, final String name)
      {
        final Component component = findComponentByName("", container, name); 
        assertNotNull("No component named " + name, component);
        return (T)component;
      }
    
   /***************************************************************************
     *
     * Finds a button given the bound action name.
     *
     **************************************************************************/
    public static <T extends Component> T findComponentByClass (final Container container, final Class<T> clazz)
      {
        final Component component = findComponentByClass("", container, clazz); 
        assertNotNull("No component of class " + clazz, component);  
        return (T)component;
      }
    
    /***************************************************************************
     *
     * Finds a button given the bound action name.
     *
     **************************************************************************/
    public static JButton findButtonByActionKey (final Container container, final String actionKey)
      {
        final Action action = Utils.findActionByKey(actionKey);
        final JButton button = findButtonByAction("", container, action.getClass().getName());
        assertNotNull("No button for " + actionKey, button);
        return button;
      }
    
    /***************************************************************************
     *
     * Finds a button given the bound action name.
     *
     **************************************************************************/
    public static JButton findButtonByAction (final Container container, final String actionName)
      {
        final JButton button = findButtonByAction("", container, actionName);
        assertNotNull("No button for " + actionName, button);
        return button;
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    public static JButton findDialogOkButton (final Component component)
      {
        final Dialog dialog = findDialog(component);
        return findOkButton(dialog);
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    public static JButton findOkButton (Component component)
      {
        final JButton button = findOkButton2(component);
        assertNotNull("Cannot find ok button!", button);
        return button;
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    public static Dialog findDialog (final Component component)
      {
        logger.fine("findDialog: " + component);
        final Container parent = component.getParent();
        logger.fine(">>>> parent: " + parent);
        assertNotNull("Could't find the dialog", parent);
        
        if (parent instanceof Dialog)
          {
            return (Dialog)parent;  
          }
        
        return findDialog(parent);
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    private static Component findComponentByName (final String prefix, final Container c, final String name)
      {
        for (Component component : c.getComponents())
          {
            String name2 = component.getClass().getName();
//            System.err.println(prefix + "/" + name2);
            
            if (name.equals(component.getName()))
              {
                return component;  
              }
            
            if (component instanceof Container)
              {
                component = findComponentByName(prefix + "/" + name2, (Container)component, name);  
                
                if (component != null)
                  {
                    return component;  
                  }
              }  
          } 
        
        return null;
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    private static Component findComponentByClass (final String prefix, final Container c, final Class clazz)
      {
        for (Component component : c.getComponents())
          {
            if (clazz.isAssignableFrom(component.getClass()))
              {
                return component;  
              }
            
            if (component instanceof Container)
              {
                component = findComponentByClass(prefix + "/" + component.getName(), (Container)component, clazz);  
                
                if (component != null)
                  {
                    return component;  
                  }
              }  
          } 
        
        return null;
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    private static JButton findButtonByAction (String prefix, Container c, String actionName)
      {
        for (Component component : c.getComponents())
          {
            String name = component.getClass().getName();
//            System.err.println(prefix + "/" + name);
            
            if (component instanceof Container)
              {
                JButton button = findButtonByAction(prefix + "/" + name, (Container)component, actionName);  
                
                if (button != null)
                  {
                    return button;  
                  }
              }  
            
            Action action = null;

            if (component instanceof JButton)
              {
                action = ((JButton)component).getAction();

                if (action == null)
                  {
                    action = (Action)((JComponent)component).getClientProperty("test.action");  
                  }

                if ((action != null) && action.getClass().getName().equals(actionName))
                  {
                    return (JButton)component;  
                  }
              }
          } 
        
        return null;
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    private static JButton findOkButton2 (Component component)
      {
        logger.finest("findOkButton: " + component);
        
        if (component instanceof JButton)
          {
            if ("Ok".equalsIgnoreCase(((JButton)component).getText()))
              {
                return (JButton)component;
              }
          }
        
        if (component instanceof JRootPane)
          {
            JButton button = findOkButton2(((JRootPane)component).getContentPane());
            
            if (button != null)
              {
                return button;  
              }
            
            button = findOkButton2(((JRootPane)component).getGlassPane());
            
            if (button != null)
              {
                return button;  
              }
          }
        
        if (component instanceof Container)
          {
//            logger.finer(">>>> components: " + Arrays.asList(((Container)component).getComponents()));
            
            for (Component childComponent : ((Container)component).getComponents())
              {
                JButton button = findOkButton2(childComponent);  

                if (button != null)
                  {
                    return button;  
                  }
              }
          }
        
        return null;
      }
  }
