/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: InputManager.java,v 8f602be640c5 2010/03/28 22:35:05 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.automatedtest;

import java.lang.reflect.InvocationTargetException;
import java.awt.AWTException;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.event.InputEvent;
import javax.swing.SwingUtilities;
import it.tidalwave.util.logging.Logger;
import static org.junit.Assert.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: InputManager.java,v 8f602be640c5 2010/03/28 22:35:05 fabrizio $
 *
 **********************************************************************************************************************/
public final class InputManager
  {
    private static final String CLASS = InputManager.class.getName();    
    private static final Logger logger = Logger.getLogger(CLASS);
    
    private static final Robot robot; 
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    static
      {
        try 
          {
            robot = new Robot();
            robot.setAutoWaitForIdle(true);
          } 
        catch (AWTException e)   
          {
            throw new ExceptionInInitializerError(e); // FIXME: right type?
          }
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    public static void moveMouse (final int x, final int y)
      {
        logger.info("moveMouse(%d, %d)", x, y);
        assertFalse(SwingUtilities.isEventDispatchThread());
//        final Point point = component.getLocationOnScreen();
//        final Dimension size = component.getSize();
        robot.mouseMove(x, y);
      }
    
    /***************************************************************************
     *
     * Centers the mouse on the given component. Must not be called from the
     * AWT Thread. 
     * 
     * @param  component  the component
     *
     **************************************************************************/
    public static void centerMouseOn (final Component component) 
      {
        centerMouseOn(component, 0, 0);
      }
    
    /***************************************************************************
     *
     * Centers the mouse on the given component. Must not be called from the
     * AWT Thread. 
     * 
     * @param  component  the component
     *
     **************************************************************************/
    public static void centerMouseOn (final Component component, final int dx, final int dy) 
      {
        logger.info("centerMouseOn(%s, %d, %d)", component.getName(), dx, dy);
        assertFalse(SwingUtilities.isEventDispatchThread());
        final Point point = component.getLocationOnScreen();
        final Dimension size = component.getSize();
        robot.mouseMove((int)(point.getX() + size.getWidth () / 2) + dx, (int)(point.getY() + size.getHeight () / 2) + dy);
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    public static void centerMouseOn (final Component component, final Rectangle rectangle)
      throws InterruptedException, InvocationTargetException
      {
        logger.info("centerMouseOn(%s, %s)", component.getName(), rectangle);
        centerMouseOn(component, rectangle.x + rectangle.width / 2, rectangle.y + rectangle.height / 2);
      }

    /***************************************************************************
     *
     * Emulates a click of the mouse. Must not be called from the
     * AWT Thread. 
     *
     **************************************************************************/
    public static void clickMouse() 
      throws InterruptedException, InvocationTargetException
      {
        logger.info("clickMouse()");
        clickMouse(InputEvent.BUTTON1_MASK);
      }
    
    /***************************************************************************
     *
     * Emulates a click of the mouse with the given button (must be one of the
     * constants defined in InputEvent). Must not be called 
     * from the AWT Thread. 
     * 
     * @param  button  the button
     *
     **************************************************************************/
    public static void clickMouse (final int button)
      throws InterruptedException, InvocationTargetException
      {
        logger.info("clickMouse(%d)", button);
        assertFalse(SwingUtilities.isEventDispatchThread()); // Robot must be called from outside EDT

        robot.mousePress(button);
        robot.delay(20);
        robot.mouseRelease(button);

        Utils.delay(100); // Give time for the event to be processed
      }

    /***************************************************************************
     *
     * @param  button  the button
     *
     **************************************************************************/
    public static void pressMouse (final int button)
      throws InterruptedException, InvocationTargetException
      {
        logger.info("pressMouse(%d)", button);
        assertFalse(SwingUtilities.isEventDispatchThread()); // Robot must be called from outside EDT
        robot.mousePress(button);
        robot.delay(20);
      }

    /***************************************************************************
     *
     * @param  button  the button
     *
     **************************************************************************/
    public static void releaseMouse (final int button)
      throws InterruptedException, InvocationTargetException
      {
        logger.info("releaseMouse(%d)", button);
        assertFalse(SwingUtilities.isEventDispatchThread()); // Robot must be called from outside EDT
        robot.mouseRelease(button);
        robot.delay(20);
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
//    @Nonnull
//    private static JFrame getMainWindow()
//      throws InterruptedException, InvocationTargetException
//      {
//        final JFrame[] result = new JFrame[1];
//
//        SwingUtilities.invokeAndWait(new Runnable()
//          {
//            public void run()
//              {
//                result[0] = EnhancedWindowManager.Locator.findEnhancedWindowManager().getMainWindow();
//              }
//          });
//
//        return result[0];
//      }
  }
