/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: AutomatedTestSelectorPane.java,v 8f602be640c5 2010/03/28 22:35:05 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.automatedtest.action;

import it.tidalwave.netbeans.dialog.DialogRunner;
import it.tidalwave.netbeans.explorer.ExplorerPanel;
import it.tidalwave.netbeans.explorer.view.EnhancedBeanTreeView;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.netbeans.windows.EnhancedWindowManager;
import javax.annotation.Nonnull;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.HashMap;
import java.util.Map;
import java.awt.BorderLayout;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import org.openide.explorer.ExplorerManager;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.Lookup;
import org.openide.util.RequestProcessor;
import it.tidalwave.netbeans.automatedtest.Utils;
import it.tidalwave.netbeans.automatedtest.AutomatedTest;
import it.tidalwave.netbeans.automatedtest.AutomatedTestNode;
import it.tidalwave.netbeans.automatedtest.AutomatedTestRunner;
import it.tidalwave.netbeans.automatedtest.AutomatedTestSuite;
import it.tidalwave.netbeans.automatedtest.TestFailedException;
import it.tidalwave.netbeans.automatedtest.annotation.TestDescription;
import org.openide.util.NbBundle;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: AutomatedTestSelectorPane.java,v 8f602be640c5 2010/03/28 22:35:05 fabrizio $
 *
 **********************************************************************************************************************/
public class AutomatedTestSelectorPane extends JPanel 
  {
    private static final long serialVersionUID = -7192787146285181398L;
    
    private ExplorerPanel explorerPanel = new ExplorerPanel();
//    private ExplorerPanel explorerPanel = new ExplorerPanel(new EnhancedBeanTreeView()); // FIXME
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    public AutomatedTestSelectorPane() 
      {
        initComponents();
        final ExplorerManager explorerManager = explorerPanel.getExplorerManager();
        explorerManager.addPropertyChangeListener(new PropertyChangeListener()
          {
            public void propertyChange (@Nonnull final PropertyChangeEvent event)
              {
                btRun.setEnabled(explorerManager.getSelectedNodes().length > 0);
              }
          });
        // FunctionBean.<Boolean>create("${em.selectedNodes.length > 0}").bind("em", explorerPanel.getExplorerManager());

        pnPane.add(explorerPanel, BorderLayout.CENTER);
//        explorerPanel.getView().setFont(pnPane.getFont()); FIXME
       
        try
          {
            Utils.getTestSetPath();
          }
        catch (Exception e)
          {
            DialogRunner.showError(e.getMessage());
            return;
          }
        
//        DialogRunner.showMessage("To run tests properly, please make sure all options about Explorers and Thumbnails are disabled");
        
        final Map<String, AutomatedTestSuite> testMap = new HashMap<String, AutomatedTestSuite>();
        final AutomatedTestSuite allTests = new AutomatedTestSuite("All tests");
        
        for (final AutomatedTest test : Lookup.getDefault().lookupAll(AutomatedTest.class))
          {          
            final String packageName = test.getClass().getPackage().getName();
            AutomatedTestSuite suite = testMap.get(packageName);
            
            if (suite == null)
              {
                suite = new AutomatedTestSuite(packageName);
                testMap.put(packageName, suite);
                allTests.add(suite);
              }
            
            suite.add(test);
          }
        
        explorerManager.setRootContext(createTestSuiteNode(allTests));
        explorerManager.addPropertyChangeListener(new PropertyChangeListener()
          {
            public void propertyChange (final PropertyChangeEvent event) 
              {
                final AutomatedTest test = explorerManager.getSelectedNodes()[0].getLookup().lookup(AutomatedTest.class);
                String name = "";

                if (test != null)
                  {
                    final TestDescription description = test.getClass().getAnnotation(TestDescription.class);  
  
                    if (description != null)
                      {
                        name = description.value();  
                      }
                  }

                lbDescription.setText("<html>" + name + "</html>");
              }
          });
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    private static AbstractNode createTestSuiteNode (final AutomatedTestSuite testSuite)
      {
        final AutomatedTestNode node = new AutomatedTestNode(new Children.Array()
          {
            @Override
            public void addNotify()
              {
                super.addNotify();

                for (final AutomatedTest test : testSuite)
                  {
                    if (test instanceof AutomatedTestSuite)
                      {
                        add(new Node[]{createTestSuiteNode((AutomatedTestSuite)test)});
                      }
                    else
                      {
                        final Node node = new AutomatedTestNode(test);
                        add(new Node[]{node});
                      }
                  }
              }
          }, testSuite);
 
        return node;
      }
    
    /***************************************************************************
     *
     *
     **************************************************************************/
    @Nonnull
    private JFrame getContainer()
      {
        return (JFrame)SwingUtilities.getAncestorOfClass(JFrame.class, this);
      }

    /***************************************************************************
     *
     * This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     *
     **************************************************************************/
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        btRun = new javax.swing.JButton();
        btClose = new javax.swing.JButton();
        pnPane = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        lbDescription = new javax.swing.JLabel();
        spRepeat = new javax.swing.JSpinner();
        lbRepeat1 = new javax.swing.JLabel();
        lbRepeat2 = new javax.swing.JLabel();
        pbRepeat = new javax.swing.JProgressBar();
        cbStopOnBrokenTest = new javax.swing.JCheckBox();

        setMinimumSize(new java.awt.Dimension(300, 400));
        setPreferredSize(new java.awt.Dimension(400, 400));

        btRun.setIcon(new javax.swing.ImageIcon(getClass().getResource("/it/tidalwave/openide/automatedtest/icons/player_play.png"))); // NOI18N
        btRun.setText("Run");
        btRun.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btRunActionPerformed(evt);
            }
        });

        btClose.setText("Close");
        btClose.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                btCloseActionPerformed(evt);
            }
        });

        pnPane.setBorder(javax.swing.BorderFactory.createTitledBorder("Select the test to run"));
        pnPane.setFont(pnPane.getFont().deriveFont(pnPane.getFont().getSize()-2f));
        pnPane.setLayout(new java.awt.BorderLayout());

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("Description"));

        lbDescription.setFont(lbDescription.getFont().deriveFont(lbDescription.getFont().getSize()-2f));
        lbDescription.setVerticalAlignment(javax.swing.SwingConstants.TOP);
        lbDescription.setVerticalTextPosition(javax.swing.SwingConstants.TOP);

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(lbDescription, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 360, Short.MAX_VALUE)
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, lbDescription, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 52, Short.MAX_VALUE)
        );

        spRepeat.setValue(1);

        lbRepeat1.setText("Repeat tests");

        lbRepeat2.setText("times");

        pbRepeat.setStringPainted(true);

        cbStopOnBrokenTest.setText("Stop on broken test");

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(pnPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 388, Short.MAX_VALUE)
                    .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                        .add(btRun)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(btClose, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 80, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(layout.createSequentialGroup()
                        .add(6, 6, 6)
                        .add(lbRepeat1)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(spRepeat, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 76, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(lbRepeat2)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(pbRepeat, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 173, Short.MAX_VALUE))
                    .add(cbStopOnBrokenTest))
                .addContainerGap())
        );

        layout.linkSize(new java.awt.Component[] {btClose, btRun}, org.jdesktop.layout.GroupLayout.HORIZONTAL);

        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(pnPane, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 166, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lbRepeat1)
                    .add(spRepeat, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 28, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(lbRepeat2)
                    .add(pbRepeat, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(cbStopOnBrokenTest)
                .add(33, 33, 33)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(btClose, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 24, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(btRun))
                .addContainerGap())
        );

        layout.linkSize(new java.awt.Component[] {btClose, btRun}, org.jdesktop.layout.GroupLayout.VERTICAL);

    }// </editor-fold>//GEN-END:initComponents

    private void btCloseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btCloseActionPerformed
        SwingUtilities.getWindowAncestor(this).dispose();
    }//GEN-LAST:event_btCloseActionPerformed

    private void btRunActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_btRunActionPerformed
        final String testRunnerTitle = NbBundle.getMessage(getClass(), "FN_TestRunnerTitle");
        final String testRunnerText = NbBundle.getMessage(getClass(), "FN_TestRunnerText");
        DialogRunner.showOneShotMessage(testRunnerText, testRunnerTitle, getClass(), "AutomatedTestWarning.v1");
        Locator.find(EnhancedWindowManager.class).getMainWindow().toFront();
        final AutomatedTest test = explorerPanel.getExplorerManager().getSelectedNodes()[0].getLookup().lookup(AutomatedTest.class);
        final int repeatCount = (Integer)spRepeat.getValue();
        pbRepeat.setMinimum(0);
        pbRepeat.setMaximum(repeatCount);
        pbRepeat.setValue(0);
        btRun.setEnabled(false);
        btClose.setEnabled(false);
        cbStopOnBrokenTest.setEnabled(false);
        final boolean stopOnBrokenTest = cbStopOnBrokenTest.isSelected();
        
        RequestProcessor.getDefault().post(new Runnable() 
          {
            public void run() 
              {
                final AutomatedTestRunner testCase = new AutomatedTestRunner();

                try
                  {
                    for (int pass = 0; pass < repeatCount; pass++)
                      {
                        testCase.runTest(test, stopOnBrokenTest);
                        final int done = pass + 1;

                        SwingUtilities.invokeLater(new Runnable()
                          {
                            public void run()
                              {
                                pbRepeat.setValue(done);
                              }
                          });
                      }
                  }
                catch (TestFailedException e)
                  {
                    // ok, stop tests; exception has been already logged
                  }

                SwingUtilities.invokeLater(new Runnable()
                  {
                    public void run()
                      {
                        btRun.setEnabled(true);
                        btClose.setEnabled(true);
                        cbStopOnBrokenTest.setEnabled(true);
                        getContainer().toFront();
                      }
                  });
              }
          });  
    }//GEN-LAST:event_btRunActionPerformed
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton btClose;
    private javax.swing.JButton btRun;
    private javax.swing.JCheckBox cbStopOnBrokenTest;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JLabel lbDescription;
    private javax.swing.JLabel lbRepeat1;
    private javax.swing.JLabel lbRepeat2;
    private javax.swing.JProgressBar pbRepeat;
    private javax.swing.JPanel pnPane;
    private javax.swing.JSpinner spRepeat;
    // End of variables declaration//GEN-END:variables
  }
