/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: RunAutomatedTestAction.java,v 8f602be640c5 2010/03/28 22:35:05 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.automatedtest.action;

import it.tidalwave.netbeans.dialog.DialogRunner;
import it.tidalwave.netbeans.util.Locator;
import it.tidalwave.netbeans.windows.EnhancedWindowManager;
import it.tidalwave.netbeans.workspacemanager.WorkspaceManager;
import java.awt.Point;
import java.io.File;
import javax.swing.JFrame;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.CallableSystemAction;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: RunAutomatedTestAction.java,v 8f602be640c5 2010/03/28 22:35:05 fabrizio $
 *
 **********************************************************************************************************************/
public final class RunAutomatedTestAction extends CallableSystemAction // FIXME EnhancedCallableSystemAction
  {
    private static final long serialVersionUID = 6041540615894324214L;
    private static final String TEST_WORKSPACE_NAME = "Test.bmw";
    
    @Override
    public void performAction()
//    public void doPerformAction() // FIXME
      {
        final File workspaceFolder = Locator.find(WorkspaceManager.class).getWorkspaceFolder();

        if (!workspaceFolder.getName().equals(TEST_WORKSPACE_NAME))
          {
            final String title = NbBundle.getMessage(getClass(), "FN_BadWorkspaceTitle");
            final String message = NbBundle.getMessage(getClass(), "FN_BadWorkspaceMessage");
            DialogRunner.showError(message, title);
          }
        else
          {
            final EnhancedWindowManager windowManager = Locator.find(EnhancedWindowManager.class);
            final JFrame container = new JFrame("Automated tests");
    //        final JDialog container = new JDialog(windowManager.getMainWindow(), "Automated Tests", false);
    //        container.setAlwaysOnTop(false);
            final AutomatedTestSelectorPane pane = new AutomatedTestSelectorPane();
            container.setLocationRelativeTo(windowManager.getMainWindow());
            container.setLocation(new Point(64, 64));
//            container.setIconImages(windowManager.getMainWindow().getIconImages()); TODO Java 6
            container.setIconImage(windowManager.getMainWindow().getIconImage());
            container.setMinimumSize(pane.getMinimumSize());
            container.setPreferredSize(pane.getPreferredSize());
            container.getContentPane().add(pane);
            container.pack();
            container.setVisible(true);
          }
      }

    @Override
    public String getName()
      {
        return "RunAutomatedTestAction";
      }

    @Override
    public HelpCtx getHelpCtx()
      {
        return HelpCtx.DEFAULT_HELP;
      }
  }
