/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: AutomatedTestOptionsPanelController.java,v 8f602be640c5 2010/03/28 22:35:05 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.automatedtest.preferences;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import javax.swing.JComponent;
import org.netbeans.spi.options.OptionsPanelController;
import org.openide.util.HelpCtx;
import org.openide.util.Lookup;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: AutomatedTestOptionsPanelController.java,v 8f602be640c5 2010/03/28 22:35:05 fabrizio $
 *
 **********************************************************************************************************************/
final class AutomatedTestOptionsPanelController extends OptionsPanelController 
  {
    private AutomatedTestPanel panel;
    private final PropertyChangeSupport pcs = new PropertyChangeSupport(this);
    private boolean changed;
    
    public void update() 
      {
        getPanel().load();
        changed = false;
      }
    
    public void applyChanges() 
      {
        getPanel().store();
        changed = false;
      }
    
    public void cancel() 
      {
        // need not do anything special, if no changes have been persisted yet
      }
    
    public boolean isValid() 
      {
        return getPanel().valid();
      }
    
    public boolean isChanged() 
      {
        return changed;
      }
    
    public HelpCtx getHelpCtx() 
      {
        return null; // new HelpCtx("...ID") if you have a help set
      }
    
    public JComponent getComponent (Lookup masterLookup) 
      {
        return getPanel();
      }
    
    public void addPropertyChangeListener (PropertyChangeListener l) 
      {
        pcs.addPropertyChangeListener(l);
      }
    
    public void removePropertyChangeListener (PropertyChangeListener l) 
      {
        pcs.removePropertyChangeListener(l);
      }
    
    private AutomatedTestPanel getPanel() 
      {
        if (panel == null) 
          {            
            panel = new AutomatedTestPanel(this);
          }
        
        return panel;
      }
    
    void changed() 
      {
        if (!changed) 
          {
            changed = true;
            pcs.firePropertyChange(OptionsPanelController.PROP_CHANGED, false, true);
          }
        
        pcs.firePropertyChange(OptionsPanelController.PROP_VALID, null, null);
      }   
  }
