/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: WorkspaceManager.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.workspacemanager;

import javax.annotation.Nonnull;
import javax.annotation.concurrent.ThreadSafe;
import java.util.Properties;
import java.io.File;

/***********************************************************************************************************************
 * 
 * The WorkspaceManager is able to create and switch the current workspace.
 *
 * @author Fabrizio Giudici
 * @version $Id: WorkspaceManager.java,v f17245cd751a 2010/03/14 17:14:28 fabrizio $
 *
 **********************************************************************************************************************/
@ThreadSafe
public interface WorkspaceManager
  {
    /*******************************************************************************************************************
     *
     * Returns the folder where the current workspace is stored.
     *
     * @return  the workspace folder
     *
     ******************************************************************************************************************/
    @Nonnull
    public File getWorkspaceFolder();

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Properties getBootProperties();
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public void commitBootProperties();
            
    /*******************************************************************************************************************
     *
     * Returns a folder that can be used as a cache.
     *
     * @param   name  the name
     * @return        the folder
     *
     ******************************************************************************************************************/
    @Nonnull
    public File getCacheFolder (@Nonnull String name);
    
    /*******************************************************************************************************************
     *
     * Returns a configuration file.
     *
     * @param   name  the name of the configuration file
     * @return        the file
     *
     ******************************************************************************************************************/
    @Nonnull
    public File getConfigFile (@Nonnull String name);
  }
